/* DO NOT EDIT - This file generated automatically by glX_server_table.py (from Mesa) script */

/*
 * (C) Copyright IBM Corporation 2005, 2006
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.  IN NO EVENT SHALL
 * IBM,
 * AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <inttypes.h>
#include "glxserver.h"
#include "glxext.h"
#include "indirect_dispatch.h"
#include "indirect_reqsize.h"
#include "indirect_table.h"

/*****************************************************************/
/* tree depth = 3 */
static const int_fast16_t Single_dispatch_tree[24] = {
    /* [0] -> opcode range [0, 256], node depth 1 */
    2,
    5,
    13,
    16,
    EMPTY_LEAF,

    /* [5] -> opcode range [0, 64], node depth 2 */
    2,
    LEAF(0),
    LEAF(16),
    10,
    EMPTY_LEAF,

    /* [10] -> opcode range [32, 48], node depth 3 */
    1,
    LEAF(32),
    EMPTY_LEAF,

    /* [13] -> opcode range [64, 128], node depth 2 */
    1,
    EMPTY_LEAF,
    LEAF(40),

    /* [16] -> opcode range [128, 192], node depth 2 */
    2,
    LEAF(72),
    LEAF(88),
    21,
    EMPTY_LEAF,

    /* [21] -> opcode range [160, 176], node depth 3 */
    1,
    LEAF(104),
    EMPTY_LEAF,

};

static const void *Single_function_table[112][2] = {
    /* [  0] =     0 */ {NULL, NULL},
    /* [  1] =     1 */ {__glXDisp_Render, __glXDispSwap_Render},
    /* [  2] =     2 */ {__glXDisp_RenderLarge, __glXDispSwap_RenderLarge},
    /* [  3] =     3 */ {__glXDisp_CreateContext, __glXDispSwap_CreateContext},
    /* [  4] =     4 */ {__glXDisp_DestroyContext,
                         __glXDispSwap_DestroyContext},
    /* [  5] =     5 */ {__glXDisp_MakeCurrent, __glXDispSwap_MakeCurrent},
    /* [  6] =     6 */ {__glXDisp_IsDirect, __glXDispSwap_IsDirect},
    /* [  7] =     7 */ {__glXDisp_QueryVersion, __glXDispSwap_QueryVersion},
    /* [  8] =     8 */ {__glXDisp_WaitGL, __glXDispSwap_WaitGL},
    /* [  9] =     9 */ {__glXDisp_WaitX, __glXDispSwap_WaitX},
    /* [ 10] =    10 */ {__glXDisp_CopyContext, __glXDispSwap_CopyContext},
    /* [ 11] =    11 */ {__glXDisp_SwapBuffers, __glXDispSwap_SwapBuffers},
    /* [ 12] =    12 */ {__glXDisp_UseXFont, __glXDispSwap_UseXFont},
    /* [ 13] =    13 */ {__glXDisp_CreateGLXPixmap,
                         __glXDispSwap_CreateGLXPixmap},
    /* [ 14] =    14 */ {__glXDisp_GetVisualConfigs,
                         __glXDispSwap_GetVisualConfigs},
    /* [ 15] =    15 */ {__glXDisp_DestroyGLXPixmap,
                         __glXDispSwap_DestroyGLXPixmap},
    /* [ 16] =    16 */ {__glXDisp_VendorPrivate, __glXDispSwap_VendorPrivate},
    /* [ 17] =    17 */ {__glXDisp_VendorPrivateWithReply,
                         __glXDispSwap_VendorPrivateWithReply},
    /* [ 18] =    18 */ {__glXDisp_QueryExtensionsString,
                         __glXDispSwap_QueryExtensionsString},
    /* [ 19] =    19 */ {__glXDisp_QueryServerString,
                         __glXDispSwap_QueryServerString},
    /* [ 20] =    20 */ {__glXDisp_ClientInfo, __glXDispSwap_ClientInfo},
    /* [ 21] =    21 */ {__glXDisp_GetFBConfigs, __glXDispSwap_GetFBConfigs},
    /* [ 22] =    22 */ {__glXDisp_CreatePixmap, __glXDispSwap_CreatePixmap},
    /* [ 23] =    23 */ {__glXDisp_DestroyPixmap, __glXDispSwap_DestroyPixmap},
    /* [ 24] =    24 */ {__glXDisp_CreateNewContext,
                         __glXDispSwap_CreateNewContext},
    /* [ 25] =    25 */ {__glXDisp_QueryContext, __glXDispSwap_QueryContext},
    /* [ 26] =    26 */ {__glXDisp_MakeContextCurrent,
                         __glXDispSwap_MakeContextCurrent},
    /* [ 27] =    27 */ {__glXDisp_CreatePbuffer, __glXDispSwap_CreatePbuffer},
    /* [ 28] =    28 */ {__glXDisp_DestroyPbuffer,
                         __glXDispSwap_DestroyPbuffer},
    /* [ 29] =    29 */ {__glXDisp_GetDrawableAttributes,
                         __glXDispSwap_GetDrawableAttributes},
    /* [ 30] =    30 */ {__glXDisp_ChangeDrawableAttributes,
                         __glXDispSwap_ChangeDrawableAttributes},
    /* [ 31] =    31 */ {__glXDisp_CreateWindow, __glXDispSwap_CreateWindow},
    /* [ 32] =    32 */ {__glXDisp_DestroyWindow, __glXDispSwap_DestroyWindow},
    /* [ 33] =    33 */ {__glXDisp_SetClientInfoARB,
                         __glXDispSwap_SetClientInfoARB},
    /* [ 34] =    34 */ {__glXDisp_CreateContextAttribsARB,
                         __glXDispSwap_CreateContextAttribsARB},
    /* [ 35] =    35 */ {__glXDisp_SetClientInfo2ARB,
                         __glXDispSwap_SetClientInfo2ARB},
    /* [ 36] =    36 */ {NULL, NULL},
    /* [ 37] =    37 */ {NULL, NULL},
    /* [ 38] =    38 */ {NULL, NULL},
    /* [ 39] =    39 */ {NULL, NULL},
    /* [ 40] =    96 */ {NULL, NULL},
    /* [ 41] =    97 */ {NULL, NULL},
    /* [ 42] =    98 */ {NULL, NULL},
    /* [ 43] =    99 */ {NULL, NULL},
    /* [ 44] =   100 */ {NULL, NULL},
    /* [ 45] =   101 */ {__glXDisp_NewList, __glXDispSwap_NewList},
    /* [ 46] =   102 */ {__glXDisp_EndList, __glXDispSwap_EndList},
    /* [ 47] =   103 */ {__glXDisp_DeleteLists, __glXDispSwap_DeleteLists},
    /* [ 48] =   104 */ {__glXDisp_GenLists, __glXDispSwap_GenLists},
    /* [ 49] =   105 */ {__glXDisp_FeedbackBuffer,
                         __glXDispSwap_FeedbackBuffer},
    /* [ 50] =   106 */ {__glXDisp_SelectBuffer, __glXDispSwap_SelectBuffer},
    /* [ 51] =   107 */ {__glXDisp_RenderMode, __glXDispSwap_RenderMode},
    /* [ 52] =   108 */ {__glXDisp_Finish, __glXDispSwap_Finish},
    /* [ 53] =   109 */ {__glXDisp_PixelStoref, __glXDispSwap_PixelStoref},
    /* [ 54] =   110 */ {__glXDisp_PixelStorei, __glXDispSwap_PixelStorei},
    /* [ 55] =   111 */ {__glXDisp_ReadPixels, __glXDispSwap_ReadPixels},
    /* [ 56] =   112 */ {__glXDisp_GetBooleanv, __glXDispSwap_GetBooleanv},
    /* [ 57] =   113 */ {__glXDisp_GetClipPlane, __glXDispSwap_GetClipPlane},
    /* [ 58] =   114 */ {__glXDisp_GetDoublev, __glXDispSwap_GetDoublev},
    /* [ 59] =   115 */ {__glXDisp_GetError, __glXDispSwap_GetError},
    /* [ 60] =   116 */ {__glXDisp_GetFloatv, __glXDispSwap_GetFloatv},
    /* [ 61] =   117 */ {__glXDisp_GetIntegerv, __glXDispSwap_GetIntegerv},
    /* [ 62] =   118 */ {__glXDisp_GetLightfv, __glXDispSwap_GetLightfv},
    /* [ 63] =   119 */ {__glXDisp_GetLightiv, __glXDispSwap_GetLightiv},
    /* [ 64] =   120 */ {__glXDisp_GetMapdv, __glXDispSwap_GetMapdv},
    /* [ 65] =   121 */ {__glXDisp_GetMapfv, __glXDispSwap_GetMapfv},
    /* [ 66] =   122 */ {__glXDisp_GetMapiv, __glXDispSwap_GetMapiv},
    /* [ 67] =   123 */ {__glXDisp_GetMaterialfv, __glXDispSwap_GetMaterialfv},
    /* [ 68] =   124 */ {__glXDisp_GetMaterialiv, __glXDispSwap_GetMaterialiv},
    /* [ 69] =   125 */ {__glXDisp_GetPixelMapfv, __glXDispSwap_GetPixelMapfv},
    /* [ 70] =   126 */ {__glXDisp_GetPixelMapuiv,
                         __glXDispSwap_GetPixelMapuiv},
    /* [ 71] =   127 */ {__glXDisp_GetPixelMapusv,
                         __glXDispSwap_GetPixelMapusv},
    /* [ 72] =   128 */ {__glXDisp_GetPolygonStipple,
                         __glXDispSwap_GetPolygonStipple},
    /* [ 73] =   129 */ {__glXDisp_GetString, __glXDispSwap_GetString},
    /* [ 74] =   130 */ {__glXDisp_GetTexEnvfv, __glXDispSwap_GetTexEnvfv},
    /* [ 75] =   131 */ {__glXDisp_GetTexEnviv, __glXDispSwap_GetTexEnviv},
    /* [ 76] =   132 */ {__glXDisp_GetTexGendv, __glXDispSwap_GetTexGendv},
    /* [ 77] =   133 */ {__glXDisp_GetTexGenfv, __glXDispSwap_GetTexGenfv},
    /* [ 78] =   134 */ {__glXDisp_GetTexGeniv, __glXDispSwap_GetTexGeniv},
    /* [ 79] =   135 */ {__glXDisp_GetTexImage, __glXDispSwap_GetTexImage},
    /* [ 80] =   136 */ {__glXDisp_GetTexParameterfv,
                         __glXDispSwap_GetTexParameterfv},
    /* [ 81] =   137 */ {__glXDisp_GetTexParameteriv,
                         __glXDispSwap_GetTexParameteriv},
    /* [ 82] =   138 */ {__glXDisp_GetTexLevelParameterfv,
                         __glXDispSwap_GetTexLevelParameterfv},
    /* [ 83] =   139 */ {__glXDisp_GetTexLevelParameteriv,
                         __glXDispSwap_GetTexLevelParameteriv},
    /* [ 84] =   140 */ {__glXDisp_IsEnabled, __glXDispSwap_IsEnabled},
    /* [ 85] =   141 */ {__glXDisp_IsList, __glXDispSwap_IsList},
    /* [ 86] =   142 */ {__glXDisp_Flush, __glXDispSwap_Flush},
    /* [ 87] =   143 */ {__glXDisp_AreTexturesResident,
                         __glXDispSwap_AreTexturesResident},
    /* [ 88] =   144 */ {__glXDisp_DeleteTextures,
                         __glXDispSwap_DeleteTextures},
    /* [ 89] =   145 */ {__glXDisp_GenTextures, __glXDispSwap_GenTextures},
    /* [ 90] =   146 */ {__glXDisp_IsTexture, __glXDispSwap_IsTexture},
    /* [ 91] =   147 */ {__glXDisp_GetColorTable, __glXDispSwap_GetColorTable},
    /* [ 92] =   148 */ {__glXDisp_GetColorTableParameterfv,
                         __glXDispSwap_GetColorTableParameterfv},
    /* [ 93] =   149 */ {__glXDisp_GetColorTableParameteriv,
                         __glXDispSwap_GetColorTableParameteriv},
    /* [ 94] =   150 */ {__glXDisp_GetConvolutionFilter,
                         __glXDispSwap_GetConvolutionFilter},
    /* [ 95] =   151 */ {__glXDisp_GetConvolutionParameterfv,
                         __glXDispSwap_GetConvolutionParameterfv},
    /* [ 96] =   152 */ {__glXDisp_GetConvolutionParameteriv,
                         __glXDispSwap_GetConvolutionParameteriv},
    /* [ 97] =   153 */ {__glXDisp_GetSeparableFilter,
                         __glXDispSwap_GetSeparableFilter},
    /* [ 98] =   154 */ {__glXDisp_GetHistogram, __glXDispSwap_GetHistogram},
    /* [ 99] =   155 */ {__glXDisp_GetHistogramParameterfv,
                         __glXDispSwap_GetHistogramParameterfv},
    /* [ 100] =   156 */ {__glXDisp_GetHistogramParameteriv,
                          __glXDispSwap_GetHistogramParameteriv},
    /* [ 101] =   157 */ {__glXDisp_GetMinmax, __glXDispSwap_GetMinmax},
    /* [ 102] =   158 */ {__glXDisp_GetMinmaxParameterfv,
                          __glXDispSwap_GetMinmaxParameterfv},
    /* [ 103] =   159 */ {__glXDisp_GetMinmaxParameteriv,
                          __glXDispSwap_GetMinmaxParameteriv},
    /* [ 104] =   160 */ {__glXDisp_GetCompressedTexImage,
                          __glXDispSwap_GetCompressedTexImage},
    /* [ 105] =   161 */ {__glXDisp_DeleteQueries, __glXDispSwap_DeleteQueries},
    /* [ 106] =   162 */ {__glXDisp_GenQueries, __glXDispSwap_GenQueries},
    /* [ 107] =   163 */ {__glXDisp_IsQuery, __glXDispSwap_IsQuery},
    /* [ 108] =   164 */ {__glXDisp_GetQueryiv, __glXDispSwap_GetQueryiv},
    /* [ 109] =   165 */ {__glXDisp_GetQueryObjectiv,
                          __glXDispSwap_GetQueryObjectiv},
    /* [ 110] =   166 */ {__glXDisp_GetQueryObjectuiv,
                          __glXDispSwap_GetQueryObjectuiv},
    /* [ 111] =   167 */ {NULL, NULL},
};

const struct __glXDispatchInfo Single_dispatch_info = {
    8,
    Single_dispatch_tree,
    Single_function_table,
    NULL,
    NULL
};

/*****************************************************************/
/* tree depth = 8 */
static const int_fast16_t Render_dispatch_tree[92] = {
    /* [0] -> opcode range [0, 8192], node depth 1 */
    2,
    5,
    31,
    54,
    EMPTY_LEAF,

    /* [5] -> opcode range [0, 2048], node depth 2 */
    1,
    8,
    EMPTY_LEAF,

    /* [8] -> opcode range [0, 1024], node depth 3 */
    1,
    11,
    EMPTY_LEAF,

    /* [11] -> opcode range [0, 512], node depth 4 */
    1,
    14,
    EMPTY_LEAF,

    /* [14] -> opcode range [0, 256], node depth 5 */
    4,
    LEAF(0),
    LEAF(16),
    LEAF(32),
    LEAF(48),
    LEAF(64),
    LEAF(80),
    LEAF(96),
    LEAF(112),
    LEAF(128),
    LEAF(144),
    LEAF(160),
    LEAF(176),
    LEAF(192),
    LEAF(208),
    LEAF(224),
    EMPTY_LEAF,

    /* [31] -> opcode range [2048, 4096], node depth 2 */
    1,
    34,
    EMPTY_LEAF,

    /* [34] -> opcode range [2048, 3072], node depth 3 */
    1,
    37,
    EMPTY_LEAF,

    /* [37] -> opcode range [2048, 2560], node depth 4 */
    1,
    40,
    EMPTY_LEAF,

    /* [40] -> opcode range [2048, 2304], node depth 5 */
    1,
    43,
    EMPTY_LEAF,

    /* [43] -> opcode range [2048, 2176], node depth 6 */
    1,
    46,
    EMPTY_LEAF,

    /* [46] -> opcode range [2048, 2112], node depth 7 */
    1,
    49,
    EMPTY_LEAF,

    /* [49] -> opcode range [2048, 2080], node depth 8 */
    2,
    LEAF(240),
    LEAF(248),
    LEAF(256),
    EMPTY_LEAF,

    /* [54] -> opcode range [4096, 6144], node depth 2 */
    1,
    57,
    EMPTY_LEAF,

    /* [57] -> opcode range [4096, 5120], node depth 3 */
    1,
    60,
    EMPTY_LEAF,

    /* [60] -> opcode range [4096, 4608], node depth 4 */
    1,
    63,
    EMPTY_LEAF,

    /* [63] -> opcode range [4096, 4352], node depth 5 */
    4,
    LEAF(264),
    LEAF(280),
    80,
    EMPTY_LEAF,
    EMPTY_LEAF,
    LEAF(296),
    LEAF(312),
    LEAF(328),
    LEAF(344),
    EMPTY_LEAF,
    83,
    86,
    EMPTY_LEAF,
    89,
    LEAF(360),
    EMPTY_LEAF,

    /* [80] -> opcode range [4128, 4144], node depth 6 */
    1,
    LEAF(376),
    EMPTY_LEAF,

    /* [83] -> opcode range [4256, 4272], node depth 6 */
    1,
    EMPTY_LEAF,
    LEAF(384),

    /* [86] -> opcode range [4272, 4288], node depth 6 */
    1,
    LEAF(392),
    EMPTY_LEAF,

    /* [89] -> opcode range [4304, 4320], node depth 6 */
    1,
    EMPTY_LEAF,
    LEAF(400),

};

static const void *Render_function_table[408][2] = {
    /* [  0] =     0 */ {NULL, NULL},
    /* [  1] =     1 */ {__glXDisp_CallList, __glXDispSwap_CallList},
    /* [  2] =     2 */ {__glXDisp_CallLists, __glXDispSwap_CallLists},
    /* [  3] =     3 */ {__glXDisp_ListBase, __glXDispSwap_ListBase},
    /* [  4] =     4 */ {__glXDisp_Begin, __glXDispSwap_Begin},
    /* [  5] =     5 */ {__glXDisp_Bitmap, __glXDispSwap_Bitmap},
    /* [  6] =     6 */ {__glXDisp_Color3bv, __glXDispSwap_Color3bv},
    /* [  7] =     7 */ {__glXDisp_Color3dv, __glXDispSwap_Color3dv},
    /* [  8] =     8 */ {__glXDisp_Color3fv, __glXDispSwap_Color3fv},
    /* [  9] =     9 */ {__glXDisp_Color3iv, __glXDispSwap_Color3iv},
    /* [ 10] =    10 */ {__glXDisp_Color3sv, __glXDispSwap_Color3sv},
    /* [ 11] =    11 */ {__glXDisp_Color3ubv, __glXDispSwap_Color3ubv},
    /* [ 12] =    12 */ {__glXDisp_Color3uiv, __glXDispSwap_Color3uiv},
    /* [ 13] =    13 */ {__glXDisp_Color3usv, __glXDispSwap_Color3usv},
    /* [ 14] =    14 */ {__glXDisp_Color4bv, __glXDispSwap_Color4bv},
    /* [ 15] =    15 */ {__glXDisp_Color4dv, __glXDispSwap_Color4dv},
    /* [ 16] =    16 */ {__glXDisp_Color4fv, __glXDispSwap_Color4fv},
    /* [ 17] =    17 */ {__glXDisp_Color4iv, __glXDispSwap_Color4iv},
    /* [ 18] =    18 */ {__glXDisp_Color4sv, __glXDispSwap_Color4sv},
    /* [ 19] =    19 */ {__glXDisp_Color4ubv, __glXDispSwap_Color4ubv},
    /* [ 20] =    20 */ {__glXDisp_Color4uiv, __glXDispSwap_Color4uiv},
    /* [ 21] =    21 */ {__glXDisp_Color4usv, __glXDispSwap_Color4usv},
    /* [ 22] =    22 */ {__glXDisp_EdgeFlagv, __glXDispSwap_EdgeFlagv},
    /* [ 23] =    23 */ {__glXDisp_End, __glXDispSwap_End},
    /* [ 24] =    24 */ {__glXDisp_Indexdv, __glXDispSwap_Indexdv},
    /* [ 25] =    25 */ {__glXDisp_Indexfv, __glXDispSwap_Indexfv},
    /* [ 26] =    26 */ {__glXDisp_Indexiv, __glXDispSwap_Indexiv},
    /* [ 27] =    27 */ {__glXDisp_Indexsv, __glXDispSwap_Indexsv},
    /* [ 28] =    28 */ {__glXDisp_Normal3bv, __glXDispSwap_Normal3bv},
    /* [ 29] =    29 */ {__glXDisp_Normal3dv, __glXDispSwap_Normal3dv},
    /* [ 30] =    30 */ {__glXDisp_Normal3fv, __glXDispSwap_Normal3fv},
    /* [ 31] =    31 */ {__glXDisp_Normal3iv, __glXDispSwap_Normal3iv},
    /* [ 32] =    32 */ {__glXDisp_Normal3sv, __glXDispSwap_Normal3sv},
    /* [ 33] =    33 */ {__glXDisp_RasterPos2dv, __glXDispSwap_RasterPos2dv},
    /* [ 34] =    34 */ {__glXDisp_RasterPos2fv, __glXDispSwap_RasterPos2fv},
    /* [ 35] =    35 */ {__glXDisp_RasterPos2iv, __glXDispSwap_RasterPos2iv},
    /* [ 36] =    36 */ {__glXDisp_RasterPos2sv, __glXDispSwap_RasterPos2sv},
    /* [ 37] =    37 */ {__glXDisp_RasterPos3dv, __glXDispSwap_RasterPos3dv},
    /* [ 38] =    38 */ {__glXDisp_RasterPos3fv, __glXDispSwap_RasterPos3fv},
    /* [ 39] =    39 */ {__glXDisp_RasterPos3iv, __glXDispSwap_RasterPos3iv},
    /* [ 40] =    40 */ {__glXDisp_RasterPos3sv, __glXDispSwap_RasterPos3sv},
    /* [ 41] =    41 */ {__glXDisp_RasterPos4dv, __glXDispSwap_RasterPos4dv},
    /* [ 42] =    42 */ {__glXDisp_RasterPos4fv, __glXDispSwap_RasterPos4fv},
    /* [ 43] =    43 */ {__glXDisp_RasterPos4iv, __glXDispSwap_RasterPos4iv},
    /* [ 44] =    44 */ {__glXDisp_RasterPos4sv, __glXDispSwap_RasterPos4sv},
    /* [ 45] =    45 */ {__glXDisp_Rectdv, __glXDispSwap_Rectdv},
    /* [ 46] =    46 */ {__glXDisp_Rectfv, __glXDispSwap_Rectfv},
    /* [ 47] =    47 */ {__glXDisp_Rectiv, __glXDispSwap_Rectiv},
    /* [ 48] =    48 */ {__glXDisp_Rectsv, __glXDispSwap_Rectsv},
    /* [ 49] =    49 */ {__glXDisp_TexCoord1dv, __glXDispSwap_TexCoord1dv},
    /* [ 50] =    50 */ {__glXDisp_TexCoord1fv, __glXDispSwap_TexCoord1fv},
    /* [ 51] =    51 */ {__glXDisp_TexCoord1iv, __glXDispSwap_TexCoord1iv},
    /* [ 52] =    52 */ {__glXDisp_TexCoord1sv, __glXDispSwap_TexCoord1sv},
    /* [ 53] =    53 */ {__glXDisp_TexCoord2dv, __glXDispSwap_TexCoord2dv},
    /* [ 54] =    54 */ {__glXDisp_TexCoord2fv, __glXDispSwap_TexCoord2fv},
    /* [ 55] =    55 */ {__glXDisp_TexCoord2iv, __glXDispSwap_TexCoord2iv},
    /* [ 56] =    56 */ {__glXDisp_TexCoord2sv, __glXDispSwap_TexCoord2sv},
    /* [ 57] =    57 */ {__glXDisp_TexCoord3dv, __glXDispSwap_TexCoord3dv},
    /* [ 58] =    58 */ {__glXDisp_TexCoord3fv, __glXDispSwap_TexCoord3fv},
    /* [ 59] =    59 */ {__glXDisp_TexCoord3iv, __glXDispSwap_TexCoord3iv},
    /* [ 60] =    60 */ {__glXDisp_TexCoord3sv, __glXDispSwap_TexCoord3sv},
    /* [ 61] =    61 */ {__glXDisp_TexCoord4dv, __glXDispSwap_TexCoord4dv},
    /* [ 62] =    62 */ {__glXDisp_TexCoord4fv, __glXDispSwap_TexCoord4fv},
    /* [ 63] =    63 */ {__glXDisp_TexCoord4iv, __glXDispSwap_TexCoord4iv},
    /* [ 64] =    64 */ {__glXDisp_TexCoord4sv, __glXDispSwap_TexCoord4sv},
    /* [ 65] =    65 */ {__glXDisp_Vertex2dv, __glXDispSwap_Vertex2dv},
    /* [ 66] =    66 */ {__glXDisp_Vertex2fv, __glXDispSwap_Vertex2fv},
    /* [ 67] =    67 */ {__glXDisp_Vertex2iv, __glXDispSwap_Vertex2iv},
    /* [ 68] =    68 */ {__glXDisp_Vertex2sv, __glXDispSwap_Vertex2sv},
    /* [ 69] =    69 */ {__glXDisp_Vertex3dv, __glXDispSwap_Vertex3dv},
    /* [ 70] =    70 */ {__glXDisp_Vertex3fv, __glXDispSwap_Vertex3fv},
    /* [ 71] =    71 */ {__glXDisp_Vertex3iv, __glXDispSwap_Vertex3iv},
    /* [ 72] =    72 */ {__glXDisp_Vertex3sv, __glXDispSwap_Vertex3sv},
    /* [ 73] =    73 */ {__glXDisp_Vertex4dv, __glXDispSwap_Vertex4dv},
    /* [ 74] =    74 */ {__glXDisp_Vertex4fv, __glXDispSwap_Vertex4fv},
    /* [ 75] =    75 */ {__glXDisp_Vertex4iv, __glXDispSwap_Vertex4iv},
    /* [ 76] =    76 */ {__glXDisp_Vertex4sv, __glXDispSwap_Vertex4sv},
    /* [ 77] =    77 */ {__glXDisp_ClipPlane, __glXDispSwap_ClipPlane},
    /* [ 78] =    78 */ {__glXDisp_ColorMaterial, __glXDispSwap_ColorMaterial},
    /* [ 79] =    79 */ {__glXDisp_CullFace, __glXDispSwap_CullFace},
    /* [ 80] =    80 */ {__glXDisp_Fogf, __glXDispSwap_Fogf},
    /* [ 81] =    81 */ {__glXDisp_Fogfv, __glXDispSwap_Fogfv},
    /* [ 82] =    82 */ {__glXDisp_Fogi, __glXDispSwap_Fogi},
    /* [ 83] =    83 */ {__glXDisp_Fogiv, __glXDispSwap_Fogiv},
    /* [ 84] =    84 */ {__glXDisp_FrontFace, __glXDispSwap_FrontFace},
    /* [ 85] =    85 */ {__glXDisp_Hint, __glXDispSwap_Hint},
    /* [ 86] =    86 */ {__glXDisp_Lightf, __glXDispSwap_Lightf},
    /* [ 87] =    87 */ {__glXDisp_Lightfv, __glXDispSwap_Lightfv},
    /* [ 88] =    88 */ {__glXDisp_Lighti, __glXDispSwap_Lighti},
    /* [ 89] =    89 */ {__glXDisp_Lightiv, __glXDispSwap_Lightiv},
    /* [ 90] =    90 */ {__glXDisp_LightModelf, __glXDispSwap_LightModelf},
    /* [ 91] =    91 */ {__glXDisp_LightModelfv, __glXDispSwap_LightModelfv},
    /* [ 92] =    92 */ {__glXDisp_LightModeli, __glXDispSwap_LightModeli},
    /* [ 93] =    93 */ {__glXDisp_LightModeliv, __glXDispSwap_LightModeliv},
    /* [ 94] =    94 */ {__glXDisp_LineStipple, __glXDispSwap_LineStipple},
    /* [ 95] =    95 */ {__glXDisp_LineWidth, __glXDispSwap_LineWidth},
    /* [ 96] =    96 */ {__glXDisp_Materialf, __glXDispSwap_Materialf},
    /* [ 97] =    97 */ {__glXDisp_Materialfv, __glXDispSwap_Materialfv},
    /* [ 98] =    98 */ {__glXDisp_Materiali, __glXDispSwap_Materiali},
    /* [ 99] =    99 */ {__glXDisp_Materialiv, __glXDispSwap_Materialiv},
    /* [ 100] =   100 */ {__glXDisp_PointSize, __glXDispSwap_PointSize},
    /* [ 101] =   101 */ {__glXDisp_PolygonMode, __glXDispSwap_PolygonMode},
    /* [ 102] =   102 */ {__glXDisp_PolygonStipple,
                          __glXDispSwap_PolygonStipple},
    /* [ 103] =   103 */ {__glXDisp_Scissor, __glXDispSwap_Scissor},
    /* [ 104] =   104 */ {__glXDisp_ShadeModel, __glXDispSwap_ShadeModel},
    /* [ 105] =   105 */ {__glXDisp_TexParameterf, __glXDispSwap_TexParameterf},
    /* [ 106] =   106 */ {__glXDisp_TexParameterfv,
                          __glXDispSwap_TexParameterfv},
    /* [ 107] =   107 */ {__glXDisp_TexParameteri, __glXDispSwap_TexParameteri},
    /* [ 108] =   108 */ {__glXDisp_TexParameteriv,
                          __glXDispSwap_TexParameteriv},
    /* [ 109] =   109 */ {__glXDisp_TexImage1D, __glXDispSwap_TexImage1D},
    /* [ 110] =   110 */ {__glXDisp_TexImage2D, __glXDispSwap_TexImage2D},
    /* [ 111] =   111 */ {__glXDisp_TexEnvf, __glXDispSwap_TexEnvf},
    /* [ 112] =   112 */ {__glXDisp_TexEnvfv, __glXDispSwap_TexEnvfv},
    /* [ 113] =   113 */ {__glXDisp_TexEnvi, __glXDispSwap_TexEnvi},
    /* [ 114] =   114 */ {__glXDisp_TexEnviv, __glXDispSwap_TexEnviv},
    /* [ 115] =   115 */ {__glXDisp_TexGend, __glXDispSwap_TexGend},
    /* [ 116] =   116 */ {__glXDisp_TexGendv, __glXDispSwap_TexGendv},
    /* [ 117] =   117 */ {__glXDisp_TexGenf, __glXDispSwap_TexGenf},
    /* [ 118] =   118 */ {__glXDisp_TexGenfv, __glXDispSwap_TexGenfv},
    /* [ 119] =   119 */ {__glXDisp_TexGeni, __glXDispSwap_TexGeni},
    /* [ 120] =   120 */ {__glXDisp_TexGeniv, __glXDispSwap_TexGeniv},
    /* [ 121] =   121 */ {__glXDisp_InitNames, __glXDispSwap_InitNames},
    /* [ 122] =   122 */ {__glXDisp_LoadName, __glXDispSwap_LoadName},
    /* [ 123] =   123 */ {__glXDisp_PassThrough, __glXDispSwap_PassThrough},
    /* [ 124] =   124 */ {__glXDisp_PopName, __glXDispSwap_PopName},
    /* [ 125] =   125 */ {__glXDisp_PushName, __glXDispSwap_PushName},
    /* [ 126] =   126 */ {__glXDisp_DrawBuffer, __glXDispSwap_DrawBuffer},
    /* [ 127] =   127 */ {__glXDisp_Clear, __glXDispSwap_Clear},
    /* [ 128] =   128 */ {__glXDisp_ClearAccum, __glXDispSwap_ClearAccum},
    /* [ 129] =   129 */ {__glXDisp_ClearIndex, __glXDispSwap_ClearIndex},
    /* [ 130] =   130 */ {__glXDisp_ClearColor, __glXDispSwap_ClearColor},
    /* [ 131] =   131 */ {__glXDisp_ClearStencil, __glXDispSwap_ClearStencil},
    /* [ 132] =   132 */ {__glXDisp_ClearDepth, __glXDispSwap_ClearDepth},
    /* [ 133] =   133 */ {__glXDisp_StencilMask, __glXDispSwap_StencilMask},
    /* [ 134] =   134 */ {__glXDisp_ColorMask, __glXDispSwap_ColorMask},
    /* [ 135] =   135 */ {__glXDisp_DepthMask, __glXDispSwap_DepthMask},
    /* [ 136] =   136 */ {__glXDisp_IndexMask, __glXDispSwap_IndexMask},
    /* [ 137] =   137 */ {__glXDisp_Accum, __glXDispSwap_Accum},
    /* [ 138] =   138 */ {__glXDisp_Disable, __glXDispSwap_Disable},
    /* [ 139] =   139 */ {__glXDisp_Enable, __glXDispSwap_Enable},
    /* [ 140] =   140 */ {NULL, NULL},
    /* [ 141] =   141 */ {__glXDisp_PopAttrib, __glXDispSwap_PopAttrib},
    /* [ 142] =   142 */ {__glXDisp_PushAttrib, __glXDispSwap_PushAttrib},
    /* [ 143] =   143 */ {__glXDisp_Map1d, __glXDispSwap_Map1d},
    /* [ 144] =   144 */ {__glXDisp_Map1f, __glXDispSwap_Map1f},
    /* [ 145] =   145 */ {__glXDisp_Map2d, __glXDispSwap_Map2d},
    /* [ 146] =   146 */ {__glXDisp_Map2f, __glXDispSwap_Map2f},
    /* [ 147] =   147 */ {__glXDisp_MapGrid1d, __glXDispSwap_MapGrid1d},
    /* [ 148] =   148 */ {__glXDisp_MapGrid1f, __glXDispSwap_MapGrid1f},
    /* [ 149] =   149 */ {__glXDisp_MapGrid2d, __glXDispSwap_MapGrid2d},
    /* [ 150] =   150 */ {__glXDisp_MapGrid2f, __glXDispSwap_MapGrid2f},
    /* [ 151] =   151 */ {__glXDisp_EvalCoord1dv, __glXDispSwap_EvalCoord1dv},
    /* [ 152] =   152 */ {__glXDisp_EvalCoord1fv, __glXDispSwap_EvalCoord1fv},
    /* [ 153] =   153 */ {__glXDisp_EvalCoord2dv, __glXDispSwap_EvalCoord2dv},
    /* [ 154] =   154 */ {__glXDisp_EvalCoord2fv, __glXDispSwap_EvalCoord2fv},
    /* [ 155] =   155 */ {__glXDisp_EvalMesh1, __glXDispSwap_EvalMesh1},
    /* [ 156] =   156 */ {__glXDisp_EvalPoint1, __glXDispSwap_EvalPoint1},
    /* [ 157] =   157 */ {__glXDisp_EvalMesh2, __glXDispSwap_EvalMesh2},
    /* [ 158] =   158 */ {__glXDisp_EvalPoint2, __glXDispSwap_EvalPoint2},
    /* [ 159] =   159 */ {__glXDisp_AlphaFunc, __glXDispSwap_AlphaFunc},
    /* [ 160] =   160 */ {__glXDisp_BlendFunc, __glXDispSwap_BlendFunc},
    /* [ 161] =   161 */ {__glXDisp_LogicOp, __glXDispSwap_LogicOp},
    /* [ 162] =   162 */ {__glXDisp_StencilFunc, __glXDispSwap_StencilFunc},
    /* [ 163] =   163 */ {__glXDisp_StencilOp, __glXDispSwap_StencilOp},
    /* [ 164] =   164 */ {__glXDisp_DepthFunc, __glXDispSwap_DepthFunc},
    /* [ 165] =   165 */ {__glXDisp_PixelZoom, __glXDispSwap_PixelZoom},
    /* [ 166] =   166 */ {__glXDisp_PixelTransferf,
                          __glXDispSwap_PixelTransferf},
    /* [ 167] =   167 */ {__glXDisp_PixelTransferi,
                          __glXDispSwap_PixelTransferi},
    /* [ 168] =   168 */ {__glXDisp_PixelMapfv, __glXDispSwap_PixelMapfv},
    /* [ 169] =   169 */ {__glXDisp_PixelMapuiv, __glXDispSwap_PixelMapuiv},
    /* [ 170] =   170 */ {__glXDisp_PixelMapusv, __glXDispSwap_PixelMapusv},
    /* [ 171] =   171 */ {__glXDisp_ReadBuffer, __glXDispSwap_ReadBuffer},
    /* [ 172] =   172 */ {__glXDisp_CopyPixels, __glXDispSwap_CopyPixels},
    /* [ 173] =   173 */ {__glXDisp_DrawPixels, __glXDispSwap_DrawPixels},
    /* [ 174] =   174 */ {__glXDisp_DepthRange, __glXDispSwap_DepthRange},
    /* [ 175] =   175 */ {__glXDisp_Frustum, __glXDispSwap_Frustum},
    /* [ 176] =   176 */ {__glXDisp_LoadIdentity, __glXDispSwap_LoadIdentity},
    /* [ 177] =   177 */ {__glXDisp_LoadMatrixf, __glXDispSwap_LoadMatrixf},
    /* [ 178] =   178 */ {__glXDisp_LoadMatrixd, __glXDispSwap_LoadMatrixd},
    /* [ 179] =   179 */ {__glXDisp_MatrixMode, __glXDispSwap_MatrixMode},
    /* [ 180] =   180 */ {__glXDisp_MultMatrixf, __glXDispSwap_MultMatrixf},
    /* [ 181] =   181 */ {__glXDisp_MultMatrixd, __glXDispSwap_MultMatrixd},
    /* [ 182] =   182 */ {__glXDisp_Ortho, __glXDispSwap_Ortho},
    /* [ 183] =   183 */ {__glXDisp_PopMatrix, __glXDispSwap_PopMatrix},
    /* [ 184] =   184 */ {__glXDisp_PushMatrix, __glXDispSwap_PushMatrix},
    /* [ 185] =   185 */ {__glXDisp_Rotated, __glXDispSwap_Rotated},
    /* [ 186] =   186 */ {__glXDisp_Rotatef, __glXDispSwap_Rotatef},
    /* [ 187] =   187 */ {__glXDisp_Scaled, __glXDispSwap_Scaled},
    /* [ 188] =   188 */ {__glXDisp_Scalef, __glXDispSwap_Scalef},
    /* [ 189] =   189 */ {__glXDisp_Translated, __glXDispSwap_Translated},
    /* [ 190] =   190 */ {__glXDisp_Translatef, __glXDispSwap_Translatef},
    /* [ 191] =   191 */ {__glXDisp_Viewport, __glXDispSwap_Viewport},
    /* [ 192] =   192 */ {__glXDisp_PolygonOffset, __glXDispSwap_PolygonOffset},
    /* [ 193] =   193 */ {__glXDisp_DrawArrays, __glXDispSwap_DrawArrays},
    /* [ 194] =   194 */ {__glXDisp_Indexubv, __glXDispSwap_Indexubv},
    /* [ 195] =   195 */ {__glXDisp_ColorSubTable, __glXDispSwap_ColorSubTable},
    /* [ 196] =   196 */ {__glXDisp_CopyColorSubTable,
                          __glXDispSwap_CopyColorSubTable},
    /* [ 197] =   197 */ {__glXDisp_ActiveTexture, __glXDispSwap_ActiveTexture},
    /* [ 198] =   198 */ {__glXDisp_MultiTexCoord1dv,
                          __glXDispSwap_MultiTexCoord1dv},
    /* [ 199] =   199 */ {__glXDisp_MultiTexCoord1fvARB,
                          __glXDispSwap_MultiTexCoord1fvARB},
    /* [ 200] =   200 */ {__glXDisp_MultiTexCoord1iv,
                          __glXDispSwap_MultiTexCoord1iv},
    /* [ 201] =   201 */ {__glXDisp_MultiTexCoord1sv,
                          __glXDispSwap_MultiTexCoord1sv},
    /* [ 202] =   202 */ {__glXDisp_MultiTexCoord2dv,
                          __glXDispSwap_MultiTexCoord2dv},
    /* [ 203] =   203 */ {__glXDisp_MultiTexCoord2fvARB,
                          __glXDispSwap_MultiTexCoord2fvARB},
    /* [ 204] =   204 */ {__glXDisp_MultiTexCoord2iv,
                          __glXDispSwap_MultiTexCoord2iv},
    /* [ 205] =   205 */ {__glXDisp_MultiTexCoord2sv,
                          __glXDispSwap_MultiTexCoord2sv},
    /* [ 206] =   206 */ {__glXDisp_MultiTexCoord3dv,
                          __glXDispSwap_MultiTexCoord3dv},
    /* [ 207] =   207 */ {__glXDisp_MultiTexCoord3fvARB,
                          __glXDispSwap_MultiTexCoord3fvARB},
    /* [ 208] =   208 */ {__glXDisp_MultiTexCoord3iv,
                          __glXDispSwap_MultiTexCoord3iv},
    /* [ 209] =   209 */ {__glXDisp_MultiTexCoord3sv,
                          __glXDispSwap_MultiTexCoord3sv},
    /* [ 210] =   210 */ {__glXDisp_MultiTexCoord4dv,
                          __glXDispSwap_MultiTexCoord4dv},
    /* [ 211] =   211 */ {__glXDisp_MultiTexCoord4fvARB,
                          __glXDispSwap_MultiTexCoord4fvARB},
    /* [ 212] =   212 */ {__glXDisp_MultiTexCoord4iv,
                          __glXDispSwap_MultiTexCoord4iv},
    /* [ 213] =   213 */ {__glXDisp_MultiTexCoord4sv,
                          __glXDispSwap_MultiTexCoord4sv},
    /* [ 214] =   214 */ {__glXDisp_CompressedTexImage1D,
                          __glXDispSwap_CompressedTexImage1D},
    /* [ 215] =   215 */ {__glXDisp_CompressedTexImage2D,
                          __glXDispSwap_CompressedTexImage2D},
    /* [ 216] =   216 */ {__glXDisp_CompressedTexImage3D,
                          __glXDispSwap_CompressedTexImage3D},
    /* [ 217] =   217 */ {__glXDisp_CompressedTexSubImage1D,
                          __glXDispSwap_CompressedTexSubImage1D},
    /* [ 218] =   218 */ {__glXDisp_CompressedTexSubImage2D,
                          __glXDispSwap_CompressedTexSubImage2D},
    /* [ 219] =   219 */ {__glXDisp_CompressedTexSubImage3D,
                          __glXDispSwap_CompressedTexSubImage3D},
    /* [ 220] =   220 */ {NULL, NULL},
    /* [ 221] =   221 */ {NULL, NULL},
    /* [ 222] =   222 */ {NULL, NULL},
    /* [ 223] =   223 */ {NULL, NULL},
    /* [ 224] =   224 */ {NULL, NULL},
    /* [ 225] =   225 */ {NULL, NULL},
    /* [ 226] =   226 */ {NULL, NULL},
    /* [ 227] =   227 */ {NULL, NULL},
    /* [ 228] =   228 */ {NULL, NULL},
    /* [ 229] =   229 */ {__glXDisp_SampleCoverage,
                          __glXDispSwap_SampleCoverage},
    /* [ 230] =   230 */ {__glXDisp_WindowPos3fv, __glXDispSwap_WindowPos3fv},
    /* [ 231] =   231 */ {__glXDisp_BeginQuery, __glXDispSwap_BeginQuery},
    /* [ 232] =   232 */ {__glXDisp_EndQuery, __glXDispSwap_EndQuery},
    /* [ 233] =   233 */ {__glXDisp_DrawBuffers, __glXDispSwap_DrawBuffers},
    /* [ 234] =   234 */ {__glXDisp_ClampColor, __glXDispSwap_ClampColor},
    /* [ 235] =   235 */ {NULL, NULL},
    /* [ 236] =   236 */ {NULL, NULL},
    /* [ 237] =   237 */ {__glXDisp_FramebufferTextureLayer,
                          __glXDispSwap_FramebufferTextureLayer},
    /* [ 238] =   238 */ {NULL, NULL},
    /* [ 239] =   239 */ {NULL, NULL},
    /* [ 240] =  2048 */ {NULL, NULL},
    /* [ 241] =  2049 */ {NULL, NULL},
    /* [ 242] =  2050 */ {NULL, NULL},
    /* [ 243] =  2051 */ {NULL, NULL},
    /* [ 244] =  2052 */ {NULL, NULL},
    /* [ 245] =  2053 */ {__glXDisp_ColorTable, __glXDispSwap_ColorTable},
    /* [ 246] =  2054 */ {__glXDisp_ColorTableParameterfv,
                          __glXDispSwap_ColorTableParameterfv},
    /* [ 247] =  2055 */ {__glXDisp_ColorTableParameteriv,
                          __glXDispSwap_ColorTableParameteriv},
    /* [ 248] =  2056 */ {__glXDisp_CopyColorTable,
                          __glXDispSwap_CopyColorTable},
    /* [ 249] =  2057 */ {NULL, NULL},
    /* [ 250] =  2058 */ {NULL, NULL},
    /* [ 251] =  2059 */ {NULL, NULL},
    /* [ 252] =  2060 */ {NULL, NULL},
    /* [ 253] =  2061 */ {NULL, NULL},
    /* [ 254] =  2062 */ {NULL, NULL},
    /* [ 255] =  2063 */ {NULL, NULL},
    /* [ 256] =  2064 */ {NULL, NULL},
    /* [ 257] =  2065 */ {__glXDisp_PointParameterf,
                          __glXDispSwap_PointParameterf},
    /* [ 258] =  2066 */ {__glXDisp_PointParameterfv,
                          __glXDispSwap_PointParameterfv},
    /* [ 259] =  2067 */ {NULL, NULL},
    /* [ 260] =  2068 */ {NULL, NULL},
    /* [ 261] =  2069 */ {NULL, NULL},
    /* [ 262] =  2070 */ {NULL, NULL},
    /* [ 263] =  2071 */ {NULL, NULL},
    /* [ 264] =  4096 */ {__glXDisp_BlendColor, __glXDispSwap_BlendColor},
    /* [ 265] =  4097 */ {__glXDisp_BlendEquation, __glXDispSwap_BlendEquation},
    /* [ 266] =  4098 */ {NULL, NULL},
    /* [ 267] =  4099 */ {__glXDisp_TexSubImage1D, __glXDispSwap_TexSubImage1D},
    /* [ 268] =  4100 */ {__glXDisp_TexSubImage2D, __glXDispSwap_TexSubImage2D},
    /* [ 269] =  4101 */ {__glXDisp_ConvolutionFilter1D,
                          __glXDispSwap_ConvolutionFilter1D},
    /* [ 270] =  4102 */ {__glXDisp_ConvolutionFilter2D,
                          __glXDispSwap_ConvolutionFilter2D},
    /* [ 271] =  4103 */ {__glXDisp_ConvolutionParameterf,
                          __glXDispSwap_ConvolutionParameterf},
    /* [ 272] =  4104 */ {__glXDisp_ConvolutionParameterfv,
                          __glXDispSwap_ConvolutionParameterfv},
    /* [ 273] =  4105 */ {__glXDisp_ConvolutionParameteri,
                          __glXDispSwap_ConvolutionParameteri},
    /* [ 274] =  4106 */ {__glXDisp_ConvolutionParameteriv,
                          __glXDispSwap_ConvolutionParameteriv},
    /* [ 275] =  4107 */ {__glXDisp_CopyConvolutionFilter1D,
                          __glXDispSwap_CopyConvolutionFilter1D},
    /* [ 276] =  4108 */ {__glXDisp_CopyConvolutionFilter2D,
                          __glXDispSwap_CopyConvolutionFilter2D},
    /* [ 277] =  4109 */ {__glXDisp_SeparableFilter2D,
                          __glXDispSwap_SeparableFilter2D},
    /* [ 278] =  4110 */ {__glXDisp_Histogram, __glXDispSwap_Histogram},
    /* [ 279] =  4111 */ {__glXDisp_Minmax, __glXDispSwap_Minmax},
    /* [ 280] =  4112 */ {__glXDisp_ResetHistogram,
                          __glXDispSwap_ResetHistogram},
    /* [ 281] =  4113 */ {__glXDisp_ResetMinmax, __glXDispSwap_ResetMinmax},
    /* [ 282] =  4114 */ {__glXDisp_TexImage3D, __glXDispSwap_TexImage3D},
    /* [ 283] =  4115 */ {__glXDisp_TexSubImage3D, __glXDispSwap_TexSubImage3D},
    /* [ 284] =  4116 */ {NULL, NULL},
    /* [ 285] =  4117 */ {__glXDisp_BindTexture, __glXDispSwap_BindTexture},
    /* [ 286] =  4118 */ {__glXDisp_PrioritizeTextures,
                          __glXDispSwap_PrioritizeTextures},
    /* [ 287] =  4119 */ {__glXDisp_CopyTexImage1D,
                          __glXDispSwap_CopyTexImage1D},
    /* [ 288] =  4120 */ {__glXDisp_CopyTexImage2D,
                          __glXDispSwap_CopyTexImage2D},
    /* [ 289] =  4121 */ {__glXDisp_CopyTexSubImage1D,
                          __glXDispSwap_CopyTexSubImage1D},
    /* [ 290] =  4122 */ {__glXDisp_CopyTexSubImage2D,
                          __glXDispSwap_CopyTexSubImage2D},
    /* [ 291] =  4123 */ {__glXDisp_CopyTexSubImage3D,
                          __glXDispSwap_CopyTexSubImage3D},
    /* [ 292] =  4124 */ {__glXDisp_FogCoordfvEXT, __glXDispSwap_FogCoordfvEXT},
    /* [ 293] =  4125 */ {__glXDisp_FogCoorddv, __glXDispSwap_FogCoorddv},
    /* [ 294] =  4126 */ {__glXDisp_SecondaryColor3bv,
                          __glXDispSwap_SecondaryColor3bv},
    /* [ 295] =  4127 */ {__glXDisp_SecondaryColor3sv,
                          __glXDispSwap_SecondaryColor3sv},
    /* [ 296] =  4176 */ {NULL, NULL},
    /* [ 297] =  4177 */ {NULL, NULL},
    /* [ 298] =  4178 */ {NULL, NULL},
    /* [ 299] =  4179 */ {NULL, NULL},
    /* [ 300] =  4180 */ {__glXDisp_BindProgramARB,
                          __glXDispSwap_BindProgramARB},
    /* [ 301] =  4181 */ {NULL, NULL},
    /* [ 302] =  4182 */ {NULL, NULL},
    /* [ 303] =  4183 */ {NULL, NULL},
    /* [ 304] =  4184 */ {__glXDisp_ProgramEnvParameter4fvARB,
                          __glXDispSwap_ProgramEnvParameter4fvARB},
    /* [ 305] =  4185 */ {__glXDisp_ProgramEnvParameter4dvARB,
                          __glXDispSwap_ProgramEnvParameter4dvARB},
    /* [ 306] =  4186 */ {NULL, NULL},
    /* [ 307] =  4187 */ {NULL, NULL},
    /* [ 308] =  4188 */ {NULL, NULL},
    /* [ 309] =  4189 */ {__glXDisp_VertexAttrib1sv,
                          __glXDispSwap_VertexAttrib1sv},
    /* [ 310] =  4190 */ {__glXDisp_VertexAttrib2sv,
                          __glXDispSwap_VertexAttrib2sv},
    /* [ 311] =  4191 */ {__glXDisp_VertexAttrib3sv,
                          __glXDispSwap_VertexAttrib3sv},
    /* [ 312] =  4192 */ {__glXDisp_VertexAttrib4sv,
                          __glXDispSwap_VertexAttrib4sv},
    /* [ 313] =  4193 */ {__glXDisp_VertexAttrib1fvARB,
                          __glXDispSwap_VertexAttrib1fvARB},
    /* [ 314] =  4194 */ {__glXDisp_VertexAttrib2fvARB,
                          __glXDispSwap_VertexAttrib2fvARB},
    /* [ 315] =  4195 */ {__glXDisp_VertexAttrib3fvARB,
                          __glXDispSwap_VertexAttrib3fvARB},
    /* [ 316] =  4196 */ {__glXDisp_VertexAttrib4fvARB,
                          __glXDispSwap_VertexAttrib4fvARB},
    /* [ 317] =  4197 */ {__glXDisp_VertexAttrib1dv,
                          __glXDispSwap_VertexAttrib1dv},
    /* [ 318] =  4198 */ {__glXDisp_VertexAttrib2dv,
                          __glXDispSwap_VertexAttrib2dv},
    /* [ 319] =  4199 */ {__glXDisp_VertexAttrib3dv,
                          __glXDispSwap_VertexAttrib3dv},
    /* [ 320] =  4200 */ {__glXDisp_VertexAttrib4dv,
                          __glXDispSwap_VertexAttrib4dv},
    /* [ 321] =  4201 */ {__glXDisp_VertexAttrib4Nubv,
                          __glXDispSwap_VertexAttrib4Nubv},
    /* [ 322] =  4202 */ {__glXDisp_VertexAttribs1svNV,
                          __glXDispSwap_VertexAttribs1svNV},
    /* [ 323] =  4203 */ {__glXDisp_VertexAttribs2svNV,
                          __glXDispSwap_VertexAttribs2svNV},
    /* [ 324] =  4204 */ {__glXDisp_VertexAttribs3svNV,
                          __glXDispSwap_VertexAttribs3svNV},
    /* [ 325] =  4205 */ {__glXDisp_VertexAttribs4svNV,
                          __glXDispSwap_VertexAttribs4svNV},
    /* [ 326] =  4206 */ {__glXDisp_VertexAttribs1fvNV,
                          __glXDispSwap_VertexAttribs1fvNV},
    /* [ 327] =  4207 */ {__glXDisp_VertexAttribs2fvNV,
                          __glXDispSwap_VertexAttribs2fvNV},
    /* [ 328] =  4208 */ {__glXDisp_VertexAttribs3fvNV,
                          __glXDispSwap_VertexAttribs3fvNV},
    /* [ 329] =  4209 */ {__glXDisp_VertexAttribs4fvNV,
                          __glXDispSwap_VertexAttribs4fvNV},
    /* [ 330] =  4210 */ {__glXDisp_VertexAttribs1dvNV,
                          __glXDispSwap_VertexAttribs1dvNV},
    /* [ 331] =  4211 */ {__glXDisp_VertexAttribs2dvNV,
                          __glXDispSwap_VertexAttribs2dvNV},
    /* [ 332] =  4212 */ {__glXDisp_VertexAttribs3dvNV,
                          __glXDispSwap_VertexAttribs3dvNV},
    /* [ 333] =  4213 */ {__glXDisp_VertexAttribs4dvNV,
                          __glXDispSwap_VertexAttribs4dvNV},
    /* [ 334] =  4214 */ {__glXDisp_VertexAttribs4ubvNV,
                          __glXDispSwap_VertexAttribs4ubvNV},
    /* [ 335] =  4215 */ {__glXDisp_ProgramLocalParameter4fvARB,
                          __glXDispSwap_ProgramLocalParameter4fvARB},
    /* [ 336] =  4216 */ {__glXDisp_ProgramLocalParameter4dvARB,
                          __glXDispSwap_ProgramLocalParameter4dvARB},
    /* [ 337] =  4217 */ {__glXDisp_ProgramStringARB,
                          __glXDispSwap_ProgramStringARB},
    /* [ 338] =  4218 */ {NULL, NULL},
    /* [ 339] =  4219 */ {NULL, NULL},
    /* [ 340] =  4220 */ {__glXDisp_ActiveStencilFaceEXT,
                          __glXDispSwap_ActiveStencilFaceEXT},
    /* [ 341] =  4221 */ {__glXDisp_PointParameteri,
                          __glXDispSwap_PointParameteri},
    /* [ 342] =  4222 */ {__glXDisp_PointParameteriv,
                          __glXDispSwap_PointParameteriv},
    /* [ 343] =  4223 */ {NULL, NULL},
    /* [ 344] =  4224 */ {NULL, NULL},
    /* [ 345] =  4225 */ {NULL, NULL},
    /* [ 346] =  4226 */ {NULL, NULL},
    /* [ 347] =  4227 */ {NULL, NULL},
    /* [ 348] =  4228 */ {__glXDisp_BlendEquationSeparate,
                          __glXDispSwap_BlendEquationSeparate},
    /* [ 349] =  4229 */ {NULL, NULL},
    /* [ 350] =  4230 */ {__glXDisp_VertexAttrib4bv,
                          __glXDispSwap_VertexAttrib4bv},
    /* [ 351] =  4231 */ {__glXDisp_VertexAttrib4iv,
                          __glXDispSwap_VertexAttrib4iv},
    /* [ 352] =  4232 */ {__glXDisp_VertexAttrib4ubv,
                          __glXDispSwap_VertexAttrib4ubv},
    /* [ 353] =  4233 */ {__glXDisp_VertexAttrib4usv,
                          __glXDispSwap_VertexAttrib4usv},
    /* [ 354] =  4234 */ {__glXDisp_VertexAttrib4uiv,
                          __glXDispSwap_VertexAttrib4uiv},
    /* [ 355] =  4235 */ {__glXDisp_VertexAttrib4Nbv,
                          __glXDispSwap_VertexAttrib4Nbv},
    /* [ 356] =  4236 */ {__glXDisp_VertexAttrib4Nsv,
                          __glXDispSwap_VertexAttrib4Nsv},
    /* [ 357] =  4237 */ {__glXDisp_VertexAttrib4Niv,
                          __glXDispSwap_VertexAttrib4Niv},
    /* [ 358] =  4238 */ {__glXDisp_VertexAttrib4Nusv,
                          __glXDispSwap_VertexAttrib4Nusv},
    /* [ 359] =  4239 */ {__glXDisp_VertexAttrib4Nuiv,
                          __glXDispSwap_VertexAttrib4Nuiv},
    /* [ 360] =  4320 */ {__glXDisp_DeleteFramebuffers,
                          __glXDispSwap_DeleteFramebuffers},
    /* [ 361] =  4321 */ {__glXDisp_FramebufferTexture1D,
                          __glXDispSwap_FramebufferTexture1D},
    /* [ 362] =  4322 */ {__glXDisp_FramebufferTexture2D,
                          __glXDispSwap_FramebufferTexture2D},
    /* [ 363] =  4323 */ {__glXDisp_FramebufferTexture3D,
                          __glXDispSwap_FramebufferTexture3D},
    /* [ 364] =  4324 */ {__glXDisp_FramebufferRenderbuffer,
                          __glXDispSwap_FramebufferRenderbuffer},
    /* [ 365] =  4325 */ {__glXDisp_GenerateMipmap,
                          __glXDispSwap_GenerateMipmap},
    /* [ 366] =  4326 */ {NULL, NULL},
    /* [ 367] =  4327 */ {NULL, NULL},
    /* [ 368] =  4328 */ {NULL, NULL},
    /* [ 369] =  4329 */ {NULL, NULL},
    /* [ 370] =  4330 */ {__glXDisp_BlitFramebuffer,
                          __glXDispSwap_BlitFramebuffer},
    /* [ 371] =  4331 */ {__glXDisp_RenderbufferStorageMultisample,
                          __glXDispSwap_RenderbufferStorageMultisample},
    /* [ 372] =  4332 */ {NULL, NULL},
    /* [ 373] =  4333 */ {NULL, NULL},
    /* [ 374] =  4334 */ {NULL, NULL},
    /* [ 375] =  4335 */ {NULL, NULL},
    /* [ 376] =  4128 */ {__glXDisp_SecondaryColor3iv,
                          __glXDispSwap_SecondaryColor3iv},
    /* [ 377] =  4129 */ {__glXDisp_SecondaryColor3fvEXT,
                          __glXDispSwap_SecondaryColor3fvEXT},
    /* [ 378] =  4130 */ {__glXDisp_SecondaryColor3dv,
                          __glXDispSwap_SecondaryColor3dv},
    /* [ 379] =  4131 */ {__glXDisp_SecondaryColor3ubv,
                          __glXDispSwap_SecondaryColor3ubv},
    /* [ 380] =  4132 */ {__glXDisp_SecondaryColor3usv,
                          __glXDispSwap_SecondaryColor3usv},
    /* [ 381] =  4133 */ {__glXDisp_SecondaryColor3uiv,
                          __glXDispSwap_SecondaryColor3uiv},
    /* [ 382] =  4134 */ {__glXDisp_BlendFuncSeparate,
                          __glXDispSwap_BlendFuncSeparate},
    /* [ 383] =  4135 */ {NULL, NULL},
    /* [ 384] =  4264 */ {NULL, NULL},
    /* [ 385] =  4265 */ {__glXDisp_VertexAttrib1svNV,
                          __glXDispSwap_VertexAttrib1svNV},
    /* [ 386] =  4266 */ {__glXDisp_VertexAttrib2svNV,
                          __glXDispSwap_VertexAttrib2svNV},
    /* [ 387] =  4267 */ {__glXDisp_VertexAttrib3svNV,
                          __glXDispSwap_VertexAttrib3svNV},
    /* [ 388] =  4268 */ {__glXDisp_VertexAttrib4svNV,
                          __glXDispSwap_VertexAttrib4svNV},
    /* [ 389] =  4269 */ {__glXDisp_VertexAttrib1fvNV,
                          __glXDispSwap_VertexAttrib1fvNV},
    /* [ 390] =  4270 */ {__glXDisp_VertexAttrib2fvNV,
                          __glXDispSwap_VertexAttrib2fvNV},
    /* [ 391] =  4271 */ {__glXDisp_VertexAttrib3fvNV,
                          __glXDispSwap_VertexAttrib3fvNV},
    /* [ 392] =  4272 */ {__glXDisp_VertexAttrib4fvNV,
                          __glXDispSwap_VertexAttrib4fvNV},
    /* [ 393] =  4273 */ {__glXDisp_VertexAttrib1dvNV,
                          __glXDispSwap_VertexAttrib1dvNV},
    /* [ 394] =  4274 */ {__glXDisp_VertexAttrib2dvNV,
                          __glXDispSwap_VertexAttrib2dvNV},
    /* [ 395] =  4275 */ {__glXDisp_VertexAttrib3dvNV,
                          __glXDispSwap_VertexAttrib3dvNV},
    /* [ 396] =  4276 */ {__glXDisp_VertexAttrib4dvNV,
                          __glXDispSwap_VertexAttrib4dvNV},
    /* [ 397] =  4277 */ {__glXDisp_VertexAttrib4ubvNV,
                          __glXDispSwap_VertexAttrib4ubvNV},
    /* [ 398] =  4278 */ {NULL, NULL},
    /* [ 399] =  4279 */ {NULL, NULL},
    /* [ 400] =  4312 */ {NULL, NULL},
    /* [ 401] =  4313 */ {NULL, NULL},
    /* [ 402] =  4314 */ {NULL, NULL},
    /* [ 403] =  4315 */ {NULL, NULL},
    /* [ 404] =  4316 */ {__glXDisp_BindRenderbuffer,
                          __glXDispSwap_BindRenderbuffer},
    /* [ 405] =  4317 */ {__glXDisp_DeleteRenderbuffers,
                          __glXDispSwap_DeleteRenderbuffers},
    /* [ 406] =  4318 */ {__glXDisp_RenderbufferStorage,
                          __glXDispSwap_RenderbufferStorage},
    /* [ 407] =  4319 */ {__glXDisp_BindFramebuffer,
                          __glXDispSwap_BindFramebuffer},
};

static const int_fast16_t Render_size_table[408][2] = {
    /* [  0] =     0 */ {0, ~0},
    /* [  1] =     1 */ {8, ~0},
    /* [  2] =     2 */ {12, 0},
    /* [  3] =     3 */ {8, ~0},
    /* [  4] =     4 */ {8, ~0},
    /* [  5] =     5 */ {48, 1},
    /* [  6] =     6 */ {8, ~0},
    /* [  7] =     7 */ {28, ~0},
    /* [  8] =     8 */ {16, ~0},
    /* [  9] =     9 */ {16, ~0},
    /* [ 10] =    10 */ {12, ~0},
    /* [ 11] =    11 */ {8, ~0},
    /* [ 12] =    12 */ {16, ~0},
    /* [ 13] =    13 */ {12, ~0},
    /* [ 14] =    14 */ {8, ~0},
    /* [ 15] =    15 */ {36, ~0},
    /* [ 16] =    16 */ {20, ~0},
    /* [ 17] =    17 */ {20, ~0},
    /* [ 18] =    18 */ {12, ~0},
    /* [ 19] =    19 */ {8, ~0},
    /* [ 20] =    20 */ {20, ~0},
    /* [ 21] =    21 */ {12, ~0},
    /* [ 22] =    22 */ {8, ~0},
    /* [ 23] =    23 */ {4, ~0},
    /* [ 24] =    24 */ {12, ~0},
    /* [ 25] =    25 */ {8, ~0},
    /* [ 26] =    26 */ {8, ~0},
    /* [ 27] =    27 */ {8, ~0},
    /* [ 28] =    28 */ {8, ~0},
    /* [ 29] =    29 */ {28, ~0},
    /* [ 30] =    30 */ {16, ~0},
    /* [ 31] =    31 */ {16, ~0},
    /* [ 32] =    32 */ {12, ~0},
    /* [ 33] =    33 */ {20, ~0},
    /* [ 34] =    34 */ {12, ~0},
    /* [ 35] =    35 */ {12, ~0},
    /* [ 36] =    36 */ {8, ~0},
    /* [ 37] =    37 */ {28, ~0},
    /* [ 38] =    38 */ {16, ~0},
    /* [ 39] =    39 */ {16, ~0},
    /* [ 40] =    40 */ {12, ~0},
    /* [ 41] =    41 */ {36, ~0},
    /* [ 42] =    42 */ {20, ~0},
    /* [ 43] =    43 */ {20, ~0},
    /* [ 44] =    44 */ {12, ~0},
    /* [ 45] =    45 */ {36, ~0},
    /* [ 46] =    46 */ {20, ~0},
    /* [ 47] =    47 */ {20, ~0},
    /* [ 48] =    48 */ {12, ~0},
    /* [ 49] =    49 */ {12, ~0},
    /* [ 50] =    50 */ {8, ~0},
    /* [ 51] =    51 */ {8, ~0},
    /* [ 52] =    52 */ {8, ~0},
    /* [ 53] =    53 */ {20, ~0},
    /* [ 54] =    54 */ {12, ~0},
    /* [ 55] =    55 */ {12, ~0},
    /* [ 56] =    56 */ {8, ~0},
    /* [ 57] =    57 */ {28, ~0},
    /* [ 58] =    58 */ {16, ~0},
    /* [ 59] =    59 */ {16, ~0},
    /* [ 60] =    60 */ {12, ~0},
    /* [ 61] =    61 */ {36, ~0},
    /* [ 62] =    62 */ {20, ~0},
    /* [ 63] =    63 */ {20, ~0},
    /* [ 64] =    64 */ {12, ~0},
    /* [ 65] =    65 */ {20, ~0},
    /* [ 66] =    66 */ {12, ~0},
    /* [ 67] =    67 */ {12, ~0},
    /* [ 68] =    68 */ {8, ~0},
    /* [ 69] =    69 */ {28, ~0},
    /* [ 70] =    70 */ {16, ~0},
    /* [ 71] =    71 */ {16, ~0},
    /* [ 72] =    72 */ {12, ~0},
    /* [ 73] =    73 */ {36, ~0},
    /* [ 74] =    74 */ {20, ~0},
    /* [ 75] =    75 */ {20, ~0},
    /* [ 76] =    76 */ {12, ~0},
    /* [ 77] =    77 */ {40, ~0},
    /* [ 78] =    78 */ {12, ~0},
    /* [ 79] =    79 */ {8, ~0},
    /* [ 80] =    80 */ {12, ~0},
    /* [ 81] =    81 */ {8, 2},
    /* [ 82] =    82 */ {12, ~0},
    /* [ 83] =    83 */ {8, 3},
    /* [ 84] =    84 */ {8, ~0},
    /* [ 85] =    85 */ {12, ~0},
    /* [ 86] =    86 */ {16, ~0},
    /* [ 87] =    87 */ {12, 4},
    /* [ 88] =    88 */ {16, ~0},
    /* [ 89] =    89 */ {12, 5},
    /* [ 90] =    90 */ {12, ~0},
    /* [ 91] =    91 */ {8, 6},
    /* [ 92] =    92 */ {12, ~0},
    /* [ 93] =    93 */ {8, 7},
    /* [ 94] =    94 */ {12, ~0},
    /* [ 95] =    95 */ {8, ~0},
    /* [ 96] =    96 */ {16, ~0},
    /* [ 97] =    97 */ {12, 8},
    /* [ 98] =    98 */ {16, ~0},
    /* [ 99] =    99 */ {12, 9},
    /* [100] =   100 */ {8, ~0},
    /* [101] =   101 */ {12, ~0},
    /* [102] =   102 */ {24, 10},
    /* [103] =   103 */ {20, ~0},
    /* [104] =   104 */ {8, ~0},
    /* [105] =   105 */ {16, ~0},
    /* [106] =   106 */ {12, 11},
    /* [107] =   107 */ {16, ~0},
    /* [108] =   108 */ {12, 12},
    /* [109] =   109 */ {56, 13},
    /* [110] =   110 */ {56, 14},
    /* [111] =   111 */ {16, ~0},
    /* [112] =   112 */ {12, 15},
    /* [113] =   113 */ {16, ~0},
    /* [114] =   114 */ {12, 16},
    /* [115] =   115 */ {20, ~0},
    /* [116] =   116 */ {12, 17},
    /* [117] =   117 */ {16, ~0},
    /* [118] =   118 */ {12, 18},
    /* [119] =   119 */ {16, ~0},
    /* [120] =   120 */ {12, 19},
    /* [121] =   121 */ {4, ~0},
    /* [122] =   122 */ {8, ~0},
    /* [123] =   123 */ {8, ~0},
    /* [124] =   124 */ {4, ~0},
    /* [125] =   125 */ {8, ~0},
    /* [126] =   126 */ {8, ~0},
    /* [127] =   127 */ {8, ~0},
    /* [128] =   128 */ {20, ~0},
    /* [129] =   129 */ {8, ~0},
    /* [130] =   130 */ {20, ~0},
    /* [131] =   131 */ {8, ~0},
    /* [132] =   132 */ {12, ~0},
    /* [133] =   133 */ {8, ~0},
    /* [134] =   134 */ {8, ~0},
    /* [135] =   135 */ {8, ~0},
    /* [136] =   136 */ {8, ~0},
    /* [137] =   137 */ {12, ~0},
    /* [138] =   138 */ {8, ~0},
    /* [139] =   139 */ {8, ~0},
    /* [140] =   140 */ {0, ~0},
    /* [141] =   141 */ {4, ~0},
    /* [142] =   142 */ {8, ~0},
    /* [143] =   143 */ {28, 20},
    /* [144] =   144 */ {20, 21},
    /* [145] =   145 */ {48, 22},
    /* [146] =   146 */ {32, 23},
    /* [147] =   147 */ {24, ~0},
    /* [148] =   148 */ {16, ~0},
    /* [149] =   149 */ {44, ~0},
    /* [150] =   150 */ {28, ~0},
    /* [151] =   151 */ {12, ~0},
    /* [152] =   152 */ {8, ~0},
    /* [153] =   153 */ {20, ~0},
    /* [154] =   154 */ {12, ~0},
    /* [155] =   155 */ {16, ~0},
    /* [156] =   156 */ {8, ~0},
    /* [157] =   157 */ {24, ~0},
    /* [158] =   158 */ {12, ~0},
    /* [159] =   159 */ {12, ~0},
    /* [160] =   160 */ {12, ~0},
    /* [161] =   161 */ {8, ~0},
    /* [162] =   162 */ {16, ~0},
    /* [163] =   163 */ {16, ~0},
    /* [164] =   164 */ {8, ~0},
    /* [165] =   165 */ {12, ~0},
    /* [166] =   166 */ {12, ~0},
    /* [167] =   167 */ {12, ~0},
    /* [168] =   168 */ {12, 24},
    /* [169] =   169 */ {12, 25},
    /* [170] =   170 */ {12, 26},
    /* [171] =   171 */ {8, ~0},
    /* [172] =   172 */ {24, ~0},
    /* [173] =   173 */ {40, 27},
    /* [174] =   174 */ {20, ~0},
    /* [175] =   175 */ {52, ~0},
    /* [176] =   176 */ {4, ~0},
    /* [177] =   177 */ {68, ~0},
    /* [178] =   178 */ {132, ~0},
    /* [179] =   179 */ {8, ~0},
    /* [180] =   180 */ {68, ~0},
    /* [181] =   181 */ {132, ~0},
    /* [182] =   182 */ {52, ~0},
    /* [183] =   183 */ {4, ~0},
    /* [184] =   184 */ {4, ~0},
    /* [185] =   185 */ {36, ~0},
    /* [186] =   186 */ {20, ~0},
    /* [187] =   187 */ {28, ~0},
    /* [188] =   188 */ {16, ~0},
    /* [189] =   189 */ {28, ~0},
    /* [190] =   190 */ {16, ~0},
    /* [191] =   191 */ {20, ~0},
    /* [192] =   192 */ {12, ~0},
    /* [193] =   193 */ {16, 28},
    /* [194] =   194 */ {8, ~0},
    /* [195] =   195 */ {44, 29},
    /* [196] =   196 */ {24, ~0},
    /* [197] =   197 */ {8, ~0},
    /* [198] =   198 */ {16, ~0},
    /* [199] =   199 */ {12, ~0},
    /* [200] =   200 */ {12, ~0},
    /* [201] =   201 */ {12, ~0},
    /* [202] =   202 */ {24, ~0},
    /* [203] =   203 */ {16, ~0},
    /* [204] =   204 */ {16, ~0},
    /* [205] =   205 */ {12, ~0},
    /* [206] =   206 */ {32, ~0},
    /* [207] =   207 */ {20, ~0},
    /* [208] =   208 */ {20, ~0},
    /* [209] =   209 */ {16, ~0},
    /* [210] =   210 */ {40, ~0},
    /* [211] =   211 */ {24, ~0},
    /* [212] =   212 */ {24, ~0},
    /* [213] =   213 */ {16, ~0},
    /* [214] =   214 */ {28, 30},
    /* [215] =   215 */ {32, 31},
    /* [216] =   216 */ {36, 32},
    /* [217] =   217 */ {28, 33},
    /* [218] =   218 */ {36, 34},
    /* [219] =   219 */ {44, 35},
    /* [220] =   220 */ {0, ~0},
    /* [221] =   221 */ {0, ~0},
    /* [222] =   222 */ {0, ~0},
    /* [223] =   223 */ {0, ~0},
    /* [224] =   224 */ {0, ~0},
    /* [225] =   225 */ {0, ~0},
    /* [226] =   226 */ {0, ~0},
    /* [227] =   227 */ {0, ~0},
    /* [228] =   228 */ {0, ~0},
    /* [229] =   229 */ {12, ~0},
    /* [230] =   230 */ {16, ~0},
    /* [231] =   231 */ {12, ~0},
    /* [232] =   232 */ {8, ~0},
    /* [233] =   233 */ {8, 36},
    /* [234] =   234 */ {12, ~0},
    /* [235] =   235 */ {0, ~0},
    /* [236] =   236 */ {0, ~0},
    /* [237] =   237 */ {24, ~0},
    /* [238] =   238 */ {0, ~0},
    /* [239] =   239 */ {0, ~0},
    /* [240] =  2048 */ {0, ~0},
    /* [241] =  2049 */ {0, ~0},
    /* [242] =  2050 */ {0, ~0},
    /* [243] =  2051 */ {0, ~0},
    /* [244] =  2052 */ {0, ~0},
    /* [245] =  2053 */ {44, 37},
    /* [246] =  2054 */ {12, 38},
    /* [247] =  2055 */ {12, 39},
    /* [248] =  2056 */ {24, ~0},
    /* [249] =  2057 */ {0, ~0},
    /* [250] =  2058 */ {0, ~0},
    /* [251] =  2059 */ {0, ~0},
    /* [252] =  2060 */ {0, ~0},
    /* [253] =  2061 */ {0, ~0},
    /* [254] =  2062 */ {0, ~0},
    /* [255] =  2063 */ {0, ~0},
    /* [256] =  2064 */ {0, ~0},
    /* [257] =  2065 */ {12, ~0},
    /* [258] =  2066 */ {8, 40},
    /* [259] =  2067 */ {0, ~0},
    /* [260] =  2068 */ {0, ~0},
    /* [261] =  2069 */ {0, ~0},
    /* [262] =  2070 */ {0, ~0},
    /* [263] =  2071 */ {0, ~0},
    /* [264] =  4096 */ {20, ~0},
    /* [265] =  4097 */ {8, ~0},
    /* [266] =  4098 */ {0, ~0},
    /* [267] =  4099 */ {60, 41},
    /* [268] =  4100 */ {60, 42},
    /* [269] =  4101 */ {48, 43},
    /* [270] =  4102 */ {48, 44},
    /* [271] =  4103 */ {16, ~0},
    /* [272] =  4104 */ {12, 45},
    /* [273] =  4105 */ {16, ~0},
    /* [274] =  4106 */ {12, 46},
    /* [275] =  4107 */ {24, ~0},
    /* [276] =  4108 */ {28, ~0},
    /* [277] =  4109 */ {32, 47},
    /* [278] =  4110 */ {20, ~0},
    /* [279] =  4111 */ {16, ~0},
    /* [280] =  4112 */ {8, ~0},
    /* [281] =  4113 */ {8, ~0},
    /* [282] =  4114 */ {84, 48},
    /* [283] =  4115 */ {92, 49},
    /* [284] =  4116 */ {0, ~0},
    /* [285] =  4117 */ {12, ~0},
    /* [286] =  4118 */ {8, 50},
    /* [287] =  4119 */ {32, ~0},
    /* [288] =  4120 */ {36, ~0},
    /* [289] =  4121 */ {28, ~0},
    /* [290] =  4122 */ {36, ~0},
    /* [291] =  4123 */ {40, ~0},
    /* [292] =  4124 */ {8, ~0},
    /* [293] =  4125 */ {12, ~0},
    /* [294] =  4126 */ {8, ~0},
    /* [295] =  4127 */ {12, ~0},
    /* [296] =  4176 */ {0, ~0},
    /* [297] =  4177 */ {0, ~0},
    /* [298] =  4178 */ {0, ~0},
    /* [299] =  4179 */ {0, ~0},
    /* [300] =  4180 */ {12, ~0},
    /* [301] =  4181 */ {0, ~0},
    /* [302] =  4182 */ {0, ~0},
    /* [303] =  4183 */ {0, ~0},
    /* [304] =  4184 */ {28, ~0},
    /* [305] =  4185 */ {44, ~0},
    /* [306] =  4186 */ {0, ~0},
    /* [307] =  4187 */ {0, ~0},
    /* [308] =  4188 */ {0, ~0},
    /* [309] =  4189 */ {12, ~0},
    /* [310] =  4190 */ {12, ~0},
    /* [311] =  4191 */ {16, ~0},
    /* [312] =  4192 */ {16, ~0},
    /* [313] =  4193 */ {12, ~0},
    /* [314] =  4194 */ {16, ~0},
    /* [315] =  4195 */ {20, ~0},
    /* [316] =  4196 */ {24, ~0},
    /* [317] =  4197 */ {16, ~0},
    /* [318] =  4198 */ {24, ~0},
    /* [319] =  4199 */ {32, ~0},
    /* [320] =  4200 */ {40, ~0},
    /* [321] =  4201 */ {12, ~0},
    /* [322] =  4202 */ {12, 51},
    /* [323] =  4203 */ {12, 52},
    /* [324] =  4204 */ {12, 53},
    /* [325] =  4205 */ {12, 54},
    /* [326] =  4206 */ {12, 55},
    /* [327] =  4207 */ {12, 56},
    /* [328] =  4208 */ {12, 57},
    /* [329] =  4209 */ {12, 58},
    /* [330] =  4210 */ {12, 59},
    /* [331] =  4211 */ {12, 60},
    /* [332] =  4212 */ {12, 61},
    /* [333] =  4213 */ {12, 62},
    /* [334] =  4214 */ {12, 63},
    /* [335] =  4215 */ {28, ~0},
    /* [336] =  4216 */ {44, ~0},
    /* [337] =  4217 */ {16, 64},
    /* [338] =  4218 */ {0, ~0},
    /* [339] =  4219 */ {0, ~0},
    /* [340] =  4220 */ {8, ~0},
    /* [341] =  4221 */ {12, ~0},
    /* [342] =  4222 */ {8, 65},
    /* [343] =  4223 */ {0, ~0},
    /* [344] =  4224 */ {0, ~0},
    /* [345] =  4225 */ {0, ~0},
    /* [346] =  4226 */ {0, ~0},
    /* [347] =  4227 */ {0, ~0},
    /* [348] =  4228 */ {12, ~0},
    /* [349] =  4229 */ {0, ~0},
    /* [350] =  4230 */ {12, ~0},
    /* [351] =  4231 */ {24, ~0},
    /* [352] =  4232 */ {12, ~0},
    /* [353] =  4233 */ {16, ~0},
    /* [354] =  4234 */ {24, ~0},
    /* [355] =  4235 */ {12, ~0},
    /* [356] =  4236 */ {16, ~0},
    /* [357] =  4237 */ {24, ~0},
    /* [358] =  4238 */ {16, ~0},
    /* [359] =  4239 */ {24, ~0},
    /* [360] =  4320 */ {8, 66},
    /* [361] =  4321 */ {24, ~0},
    /* [362] =  4322 */ {24, ~0},
    /* [363] =  4323 */ {28, ~0},
    /* [364] =  4324 */ {20, ~0},
    /* [365] =  4325 */ {8, ~0},
    /* [366] =  4326 */ {0, ~0},
    /* [367] =  4327 */ {0, ~0},
    /* [368] =  4328 */ {0, ~0},
    /* [369] =  4329 */ {0, ~0},
    /* [370] =  4330 */ {44, ~0},
    /* [371] =  4331 */ {24, ~0},
    /* [372] =  4332 */ {0, ~0},
    /* [373] =  4333 */ {0, ~0},
    /* [374] =  4334 */ {0, ~0},
    /* [375] =  4335 */ {0, ~0},
    /* [376] =  4128 */ {16, ~0},
    /* [377] =  4129 */ {16, ~0},
    /* [378] =  4130 */ {28, ~0},
    /* [379] =  4131 */ {8, ~0},
    /* [380] =  4132 */ {12, ~0},
    /* [381] =  4133 */ {16, ~0},
    /* [382] =  4134 */ {20, ~0},
    /* [383] =  4135 */ {0, ~0},
    /* [384] =  4264 */ {0, ~0},
    /* [385] =  4265 */ {12, ~0},
    /* [386] =  4266 */ {12, ~0},
    /* [387] =  4267 */ {16, ~0},
    /* [388] =  4268 */ {16, ~0},
    /* [389] =  4269 */ {12, ~0},
    /* [390] =  4270 */ {16, ~0},
    /* [391] =  4271 */ {20, ~0},
    /* [392] =  4272 */ {24, ~0},
    /* [393] =  4273 */ {16, ~0},
    /* [394] =  4274 */ {24, ~0},
    /* [395] =  4275 */ {32, ~0},
    /* [396] =  4276 */ {40, ~0},
    /* [397] =  4277 */ {12, ~0},
    /* [398] =  4278 */ {0, ~0},
    /* [399] =  4279 */ {0, ~0},
    /* [400] =  4312 */ {0, ~0},
    /* [401] =  4313 */ {0, ~0},
    /* [402] =  4314 */ {0, ~0},
    /* [403] =  4315 */ {0, ~0},
    /* [404] =  4316 */ {12, ~0},
    /* [405] =  4317 */ {8, 67},
    /* [406] =  4318 */ {20, ~0},
    /* [407] =  4319 */ {12, ~0},
};

static const gl_proto_size_func Render_size_func_table[68] = {
    __glXCallListsReqSize,
    __glXBitmapReqSize,
    __glXFogfvReqSize,
    __glXFogivReqSize,
    __glXLightfvReqSize,
    __glXLightivReqSize,
    __glXLightModelfvReqSize,
    __glXLightModelivReqSize,
    __glXMaterialfvReqSize,
    __glXMaterialivReqSize,
    __glXPolygonStippleReqSize,
    __glXTexParameterfvReqSize,
    __glXTexParameterivReqSize,
    __glXTexImage1DReqSize,
    __glXTexImage2DReqSize,
    __glXTexEnvfvReqSize,
    __glXTexEnvivReqSize,
    __glXTexGendvReqSize,
    __glXTexGenfvReqSize,
    __glXTexGenivReqSize,
    __glXMap1dReqSize,
    __glXMap1fReqSize,
    __glXMap2dReqSize,
    __glXMap2fReqSize,
    __glXPixelMapfvReqSize,
    __glXPixelMapuivReqSize,
    __glXPixelMapusvReqSize,
    __glXDrawPixelsReqSize,
    __glXDrawArraysReqSize,
    __glXColorSubTableReqSize,
    __glXCompressedTexImage1DReqSize,
    __glXCompressedTexImage2DReqSize,
    __glXCompressedTexImage3DReqSize,
    __glXCompressedTexSubImage1DReqSize,
    __glXCompressedTexSubImage2DReqSize,
    __glXCompressedTexSubImage3DReqSize,
    __glXDrawBuffersReqSize,
    __glXColorTableReqSize,
    __glXColorTableParameterfvReqSize,
    __glXColorTableParameterivReqSize,
    __glXPointParameterfvReqSize,
    __glXTexSubImage1DReqSize,
    __glXTexSubImage2DReqSize,
    __glXConvolutionFilter1DReqSize,
    __glXConvolutionFilter2DReqSize,
    __glXConvolutionParameterfvReqSize,
    __glXConvolutionParameterivReqSize,
    __glXSeparableFilter2DReqSize,
    __glXTexImage3DReqSize,
    __glXTexSubImage3DReqSize,
    __glXPrioritizeTexturesReqSize,
    __glXVertexAttribs1svNVReqSize,
    __glXVertexAttribs2svNVReqSize,
    __glXVertexAttribs3svNVReqSize,
    __glXVertexAttribs4svNVReqSize,
    __glXVertexAttribs1fvNVReqSize,
    __glXVertexAttribs2fvNVReqSize,
    __glXVertexAttribs3fvNVReqSize,
    __glXVertexAttribs4fvNVReqSize,
    __glXVertexAttribs1dvNVReqSize,
    __glXVertexAttribs2dvNVReqSize,
    __glXVertexAttribs3dvNVReqSize,
    __glXVertexAttribs4dvNVReqSize,
    __glXVertexAttribs4ubvNVReqSize,
    __glXProgramStringARBReqSize,
    __glXPointParameterivReqSize,
    __glXDeleteFramebuffersReqSize,
    __glXDeleteRenderbuffersReqSize,
};

const struct __glXDispatchInfo Render_dispatch_info = {
    13,
    Render_dispatch_tree,
    Render_function_table,
    Render_size_table,
    Render_size_func_table
};

/*****************************************************************/
/* tree depth = 12 */
static const int_fast16_t VendorPriv_dispatch_tree[152] = {
    /* [0] -> opcode range [0, 131072], node depth 1 */
    2,
    5,
    EMPTY_LEAF,
    119,
    EMPTY_LEAF,

    /* [5] -> opcode range [0, 32768], node depth 2 */
    1,
    8,
    EMPTY_LEAF,

    /* [8] -> opcode range [0, 16384], node depth 3 */
    1,
    11,
    EMPTY_LEAF,

    /* [11] -> opcode range [0, 8192], node depth 4 */
    2,
    16,
    EMPTY_LEAF,
    78,
    EMPTY_LEAF,

    /* [16] -> opcode range [0, 2048], node depth 5 */
    2,
    21,
    EMPTY_LEAF,
    36,
    EMPTY_LEAF,

    /* [21] -> opcode range [0, 512], node depth 6 */
    1,
    24,
    EMPTY_LEAF,

    /* [24] -> opcode range [0, 256], node depth 7 */
    1,
    27,
    EMPTY_LEAF,

    /* [27] -> opcode range [0, 128], node depth 8 */
    1,
    30,
    EMPTY_LEAF,

    /* [30] -> opcode range [0, 64], node depth 9 */
    1,
    33,
    EMPTY_LEAF,

    /* [33] -> opcode range [0, 32], node depth 10 */
    1,
    LEAF(0),
    EMPTY_LEAF,

    /* [36] -> opcode range [1024, 1536], node depth 6 */
    2,
    41,
    EMPTY_LEAF,
    53,
    67,

    /* [41] -> opcode range [1024, 1152], node depth 7 */
    1,
    44,
    EMPTY_LEAF,

    /* [44] -> opcode range [1024, 1088], node depth 8 */
    1,
    47,
    EMPTY_LEAF,

    /* [47] -> opcode range [1024, 1056], node depth 9 */
    1,
    50,
    EMPTY_LEAF,

    /* [50] -> opcode range [1024, 1040], node depth 10 */
    1,
    LEAF(16),
    EMPTY_LEAF,

    /* [53] -> opcode range [1280, 1408], node depth 7 */
    1,
    56,
    EMPTY_LEAF,

    /* [56] -> opcode range [1280, 1344], node depth 8 */
    2,
    61,
    LEAF(24),
    EMPTY_LEAF,
    64,

    /* [61] -> opcode range [1280, 1296], node depth 9 */
    1,
    EMPTY_LEAF,
    LEAF(40),

    /* [64] -> opcode range [1328, 1344], node depth 9 */
    1,
    LEAF(48),
    EMPTY_LEAF,

    /* [67] -> opcode range [1408, 1536], node depth 7 */
    1,
    70,
    EMPTY_LEAF,

    /* [70] -> opcode range [1408, 1472], node depth 8 */
    1,
    73,
    EMPTY_LEAF,

    /* [73] -> opcode range [1408, 1440], node depth 9 */
    2,
    EMPTY_LEAF,
    LEAF(56),
    LEAF(64),
    EMPTY_LEAF,

    /* [78] -> opcode range [4096, 6144], node depth 5 */
    2,
    83,
    EMPTY_LEAF,
    101,
    EMPTY_LEAF,

    /* [83] -> opcode range [4096, 4608], node depth 6 */
    1,
    86,
    EMPTY_LEAF,

    /* [86] -> opcode range [4096, 4352], node depth 7 */
    1,
    89,
    EMPTY_LEAF,

    /* [89] -> opcode range [4096, 4224], node depth 8 */
    1,
    92,
    EMPTY_LEAF,

    /* [92] -> opcode range [4096, 4160], node depth 9 */
    1,
    95,
    EMPTY_LEAF,

    /* [95] -> opcode range [4096, 4128], node depth 10 */
    1,
    98,
    EMPTY_LEAF,

    /* [98] -> opcode range [4096, 4112], node depth 11 */
    1,
    LEAF(72),
    EMPTY_LEAF,

    /* [101] -> opcode range [5120, 5632], node depth 6 */
    1,
    104,
    EMPTY_LEAF,

    /* [104] -> opcode range [5120, 5376], node depth 7 */
    1,
    107,
    EMPTY_LEAF,

    /* [107] -> opcode range [5120, 5248], node depth 8 */
    1,
    110,
    EMPTY_LEAF,

    /* [110] -> opcode range [5120, 5184], node depth 9 */
    1,
    EMPTY_LEAF,
    113,

    /* [113] -> opcode range [5152, 5184], node depth 10 */
    1,
    116,
    EMPTY_LEAF,

    /* [116] -> opcode range [5152, 5168], node depth 11 */
    1,
    LEAF(80),
    EMPTY_LEAF,

    /* [119] -> opcode range [65536, 98304], node depth 2 */
    1,
    122,
    EMPTY_LEAF,

    /* [122] -> opcode range [65536, 81920], node depth 3 */
    1,
    125,
    EMPTY_LEAF,

    /* [125] -> opcode range [65536, 73728], node depth 4 */
    1,
    128,
    EMPTY_LEAF,

    /* [128] -> opcode range [65536, 69632], node depth 5 */
    1,
    131,
    EMPTY_LEAF,

    /* [131] -> opcode range [65536, 67584], node depth 6 */
    1,
    134,
    EMPTY_LEAF,

    /* [134] -> opcode range [65536, 66560], node depth 7 */
    1,
    137,
    EMPTY_LEAF,

    /* [137] -> opcode range [65536, 66048], node depth 8 */
    1,
    140,
    EMPTY_LEAF,

    /* [140] -> opcode range [65536, 65792], node depth 9 */
    1,
    143,
    EMPTY_LEAF,

    /* [143] -> opcode range [65536, 65664], node depth 10 */
    1,
    146,
    EMPTY_LEAF,

    /* [146] -> opcode range [65536, 65600], node depth 11 */
    1,
    149,
    EMPTY_LEAF,

    /* [149] -> opcode range [65536, 65568], node depth 12 */
    1,
    LEAF(88),
    EMPTY_LEAF,

};

static const void *VendorPriv_function_table[104][2] = {
    /* [  0] =     0 */ {NULL, NULL},
    /* [  1] =     1 */ {__glXDisp_GetConvolutionFilterEXT,
                         __glXDispSwap_GetConvolutionFilterEXT},
    /* [  2] =     2 */ {__glXDisp_GetConvolutionParameterfvEXT,
                         __glXDispSwap_GetConvolutionParameterfvEXT},
    /* [  3] =     3 */ {__glXDisp_GetConvolutionParameterivEXT,
                         __glXDispSwap_GetConvolutionParameterivEXT},
    /* [  4] =     4 */ {__glXDisp_GetSeparableFilterEXT,
                         __glXDispSwap_GetSeparableFilterEXT},
    /* [  5] =     5 */ {__glXDisp_GetHistogramEXT,
                         __glXDispSwap_GetHistogramEXT},
    /* [  6] =     6 */ {__glXDisp_GetHistogramParameterfvEXT,
                         __glXDispSwap_GetHistogramParameterfvEXT},
    /* [  7] =     7 */ {__glXDisp_GetHistogramParameterivEXT,
                         __glXDispSwap_GetHistogramParameterivEXT},
    /* [  8] =     8 */ {__glXDisp_GetMinmaxEXT, __glXDispSwap_GetMinmaxEXT},
    /* [  9] =     9 */ {__glXDisp_GetMinmaxParameterfvEXT,
                         __glXDispSwap_GetMinmaxParameterfvEXT},
    /* [ 10] =    10 */ {__glXDisp_GetMinmaxParameterivEXT,
                         __glXDispSwap_GetMinmaxParameterivEXT},
    /* [ 11] =    11 */ {__glXDisp_AreTexturesResidentEXT,
                         __glXDispSwap_AreTexturesResidentEXT},
    /* [ 12] =    12 */ {__glXDisp_DeleteTexturesEXT,
                         __glXDispSwap_DeleteTexturesEXT},
    /* [ 13] =    13 */ {__glXDisp_GenTexturesEXT,
                         __glXDispSwap_GenTexturesEXT},
    /* [ 14] =    14 */ {__glXDisp_IsTextureEXT, __glXDispSwap_IsTextureEXT},
    /* [ 15] =    15 */ {NULL, NULL},
    /* [ 16] =  1024 */ {__glXDisp_QueryContextInfoEXT,
                         __glXDispSwap_QueryContextInfoEXT},
    /* [ 17] =  1025 */ {NULL, NULL},
    /* [ 18] =  1026 */ {NULL, NULL},
    /* [ 19] =  1027 */ {NULL, NULL},
    /* [ 20] =  1028 */ {NULL, NULL},
    /* [ 21] =  1029 */ {NULL, NULL},
    /* [ 22] =  1030 */ {NULL, NULL},
    /* [ 23] =  1031 */ {NULL, NULL},
    /* [ 24] =  1296 */ {__glXDisp_GetProgramEnvParameterfvARB,
                         __glXDispSwap_GetProgramEnvParameterfvARB},
    /* [ 25] =  1297 */ {__glXDisp_GetProgramEnvParameterdvARB,
                         __glXDispSwap_GetProgramEnvParameterdvARB},
    /* [ 26] =  1298 */ {NULL, NULL},
    /* [ 27] =  1299 */ {NULL, NULL},
    /* [ 28] =  1300 */ {NULL, NULL},
    /* [ 29] =  1301 */ {NULL, NULL},
    /* [ 30] =  1302 */ {NULL, NULL},
    /* [ 31] =  1303 */ {NULL, NULL},
    /* [ 32] =  1304 */ {__glXDisp_IsProgramARB, __glXDispSwap_IsProgramARB},
    /* [ 33] =  1305 */ {__glXDisp_GetProgramLocalParameterfvARB,
                         __glXDispSwap_GetProgramLocalParameterfvARB},
    /* [ 34] =  1306 */ {__glXDisp_GetProgramLocalParameterdvARB,
                         __glXDispSwap_GetProgramLocalParameterdvARB},
    /* [ 35] =  1307 */ {__glXDisp_GetProgramivARB,
                         __glXDispSwap_GetProgramivARB},
    /* [ 36] =  1308 */ {__glXDisp_GetProgramStringARB,
                         __glXDispSwap_GetProgramStringARB},
    /* [ 37] =  1309 */ {NULL, NULL},
    /* [ 38] =  1310 */ {NULL, NULL},
    /* [ 39] =  1311 */ {NULL, NULL},
    /* [ 40] =  1288 */ {NULL, NULL},
    /* [ 41] =  1289 */ {NULL, NULL},
    /* [ 42] =  1290 */ {NULL, NULL},
    /* [ 43] =  1291 */ {NULL, NULL},
    /* [ 44] =  1292 */ {NULL, NULL},
    /* [ 45] =  1293 */ {NULL, NULL},
    /* [ 46] =  1294 */ {__glXDisp_DeleteProgramsARB,
                         __glXDispSwap_DeleteProgramsARB},
    /* [ 47] =  1295 */ {__glXDisp_GenProgramsARB,
                         __glXDispSwap_GenProgramsARB},
    /* [ 48] =  1328 */ {NULL, NULL},
    /* [ 49] =  1329 */ {NULL, NULL},
    /* [ 50] =  1330 */ {__glXDisp_BindTexImageEXT,
                         __glXDispSwap_BindTexImageEXT},
    /* [ 51] =  1331 */ {__glXDisp_ReleaseTexImageEXT,
                         __glXDispSwap_ReleaseTexImageEXT},
    /* [ 52] =  1332 */ {NULL, NULL},
    /* [ 53] =  1333 */ {NULL, NULL},
    /* [ 54] =  1334 */ {NULL, NULL},
    /* [ 55] =  1335 */ {NULL, NULL},
    /* [ 56] =  1416 */ {NULL, NULL},
    /* [ 57] =  1417 */ {NULL, NULL},
    /* [ 58] =  1418 */ {NULL, NULL},
    /* [ 59] =  1419 */ {NULL, NULL},
    /* [ 60] =  1420 */ {NULL, NULL},
    /* [ 61] =  1421 */ {NULL, NULL},
    /* [ 62] =  1422 */ {__glXDisp_IsRenderbuffer,
                         __glXDispSwap_IsRenderbuffer},
    /* [ 63] =  1423 */ {__glXDisp_GenRenderbuffers,
                         __glXDispSwap_GenRenderbuffers},
    /* [ 64] =  1424 */ {__glXDisp_GetRenderbufferParameteriv,
                         __glXDispSwap_GetRenderbufferParameteriv},
    /* [ 65] =  1425 */ {__glXDisp_IsFramebuffer, __glXDispSwap_IsFramebuffer},
    /* [ 66] =  1426 */ {__glXDisp_GenFramebuffers,
                         __glXDispSwap_GenFramebuffers},
    /* [ 67] =  1427 */ {__glXDisp_CheckFramebufferStatus,
                         __glXDispSwap_CheckFramebufferStatus},
    /* [ 68] =  1428 */ {__glXDisp_GetFramebufferAttachmentParameteriv,
                         __glXDispSwap_GetFramebufferAttachmentParameteriv},
    /* [ 69] =  1429 */ {NULL, NULL},
    /* [ 70] =  1430 */ {NULL, NULL},
    /* [ 71] =  1431 */ {NULL, NULL},
    /* [ 72] =  4096 */ {NULL, NULL},
    /* [ 73] =  4097 */ {NULL, NULL},
    /* [ 74] =  4098 */ {__glXDisp_GetColorTableSGI,
                         __glXDispSwap_GetColorTableSGI},
    /* [ 75] =  4099 */ {__glXDisp_GetColorTableParameterfvSGI,
                         __glXDispSwap_GetColorTableParameterfvSGI},
    /* [ 76] =  4100 */ {__glXDisp_GetColorTableParameterivSGI,
                         __glXDispSwap_GetColorTableParameterivSGI},
    /* [ 77] =  4101 */ {NULL, NULL},
    /* [ 78] =  4102 */ {NULL, NULL},
    /* [ 79] =  4103 */ {NULL, NULL},
    /* [ 80] =  5152 */ {NULL, NULL},
    /* [ 81] =  5153 */ {NULL, NULL},
    /* [ 82] =  5154 */ {__glXDisp_CopySubBufferMESA,
                         __glXDispSwap_CopySubBufferMESA},
    /* [ 83] =  5155 */ {NULL, NULL},
    /* [ 84] =  5156 */ {NULL, NULL},
    /* [ 85] =  5157 */ {NULL, NULL},
    /* [ 86] =  5158 */ {NULL, NULL},
    /* [ 87] =  5159 */ {NULL, NULL},
    /* [ 88] = 65536 */ {__glXDisp_SwapIntervalSGI,
                         __glXDispSwap_SwapIntervalSGI},
    /* [ 89] = 65537 */ {__glXDisp_MakeCurrentReadSGI,
                         __glXDispSwap_MakeCurrentReadSGI},
    /* [ 90] = 65538 */ {NULL, NULL},
    /* [ 91] = 65539 */ {NULL, NULL},
    /* [ 92] = 65540 */ {__glXDisp_GetFBConfigsSGIX,
                         __glXDispSwap_GetFBConfigsSGIX},
    /* [ 93] = 65541 */ {__glXDisp_CreateContextWithConfigSGIX,
                         __glXDispSwap_CreateContextWithConfigSGIX},
    /* [ 94] = 65542 */ {__glXDisp_CreateGLXPixmapWithConfigSGIX,
                         __glXDispSwap_CreateGLXPixmapWithConfigSGIX},
    /* [ 95] = 65543 */ {__glXDisp_CreateGLXPbufferSGIX,
                         __glXDispSwap_CreateGLXPbufferSGIX},
    /* [ 96] = 65544 */ {__glXDisp_DestroyGLXPbufferSGIX,
                         __glXDispSwap_DestroyGLXPbufferSGIX},
    /* [ 97] = 65545 */ {__glXDisp_ChangeDrawableAttributesSGIX,
                         __glXDispSwap_ChangeDrawableAttributesSGIX},
    /* [ 98] = 65546 */ {__glXDisp_GetDrawableAttributesSGIX,
                         __glXDispSwap_GetDrawableAttributesSGIX},
    /* [ 99] = 65547 */ {NULL, NULL},
    /* [ 100] = 65548 */ {NULL, NULL},
    /* [ 101] = 65549 */ {NULL, NULL},
    /* [ 102] = 65550 */ {NULL, NULL},
    /* [ 103] = 65551 */ {NULL, NULL},
};

const struct __glXDispatchInfo VendorPriv_dispatch_info = {
    17,
    VendorPriv_dispatch_tree,
    VendorPriv_function_table,
    NULL,
    NULL
};
